Package: PanomiR
Title: Detection of miRNAs that regulate interacting groups of pathways
Version: 1.15.0
Authors@R: c(
    person("Pourya", "Naderi", email = "pouryany@gmail.com",
           role = c("aut", "cre")),
    person("Yue Yang (Alan)", "Teo", email = "yueyang.teo@epfl.ch",
           role = c("aut")),
    person("Ilya", "Sytchev", email = "isytchev@hsph.harvard.edu",
           role = c("aut")),
    person("Winston", "Hide", email = "whide@bidmc.harvard.edu",
           role = c("aut")))
Description: PanomiR is a package to detect miRNAs that target groups of
    pathways from gene expression data. This package provides functionality 
    for generating pathway activity profiles, determining differentially
    activated pathways between user-specified conditions, determining 
    clusters of pathways via the PCxN package, and generating miRNAs targeting
    clusters of pathways. These function can be used separately or sequentially
    to analyze RNA-Seq data.
License: MIT + file LICENSE
Encoding: UTF-8
RoxygenNote: 7.1.2
Suggests: testthat (>= 3.0.0), BiocStyle, knitr, rmarkdown
Config/testthat/edition: 3
biocViews: GeneExpression, GeneSetEnrichment, GeneTarget, miRNA,
        Pathways
Imports: clusterProfiler, dplyr, forcats, GSEABase, igraph, limma,
        metap, org.Hs.eg.db, parallel, preprocessCore, RColorBrewer,
        rlang, tibble, withr, utils
Depends: R (>= 4.2.0)
URL: https://github.com/pouryany/PanomiR
BugReports: https://github.com/pouryany/PanomiR/issues
VignetteBuilder: knitr
git_url: https://git.bioconductor.org/packages/PanomiR
git_branch: devel
git_last_commit: eb18fe4
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.23
Date/Publication: 2025-10-31
NeedsCompilation: no
Packaged: 2025-10-31 23:21:21 UTC; biocbuild
Author: Pourya Naderi [aut, cre],
  Yue Yang (Alan) Teo [aut],
  Ilya Sytchev [aut],
  Winston Hide [aut]
Maintainer: Pourya Naderi <pouryany@gmail.com>

# Generated by roxygen2: do not edit by hand

export(clusterPlot)
export(differentialPathwayAnalysis)
export(getDesignMatrix)
export(linColumnFinder)
export(mappingPathwaysClusters)
export(miRNAPathwayEnrichment)
export(pathwayGeneTab)
export(pathwaySummary)
export(prioritizeMicroRNA)
export(reportEnrichment)
export(tableFromGSC)

<!-- badges: start -->

[![R-CMD-check](https://github.com/pouryany/PanomiR/workflows/R-CMD-check/badge.svg)](https://github.com/pouryany/PanomiR/actions)
[![lint](https://github.com/pouryany/PanomiR/workflows/lint/badge.svg)](https://github.com/pouryany/PanomiR/actions)
<!-- badges: end -->

## Introduction

PanomiR is a package for pathway and microRNA Analysis of gene
expression data. This document provides details about how to install and
utilize various functionality in PanomiR.

For questions, comments, and other queries, contact <pouryany@gmail.com>

## Installation

PanomiR can be accessed via Bioconductor. To install, start R and run
the following code.

``` r
if (!requireNamespace("BiocManager", quietly = TRUE))
    install.packages("BiocManager")

BiocManager::install("PanomiR")
```

You can also install the latest development version of PanomiR using
GitHub.

``` r
devtools::install_github("pouryany/PanomiR")
```

## Overview

PanomiR is a pipeline to prioritize disease-associated miRNAs based on
activity of disease-associated pathways. The input datasets for PanomiR
are (a) a gene expression disease dataset along with covariates, (b) a
background collection of pathways/genesets, and (c) a collection of
miRNAs containing gene targets.

The general workflow of PanomiR is (a) generation of pathway summary
statistics from gene expression data, (b) detection of differentially
activated pathways, (c) finding coherent groups, or clusters, of
differentially activated pathways, and (d) detecting miRNAs targeting
each group of pathways.

Individual steps of the workflow can be used in isolation to carry out
different analyses. The following sections outline each step and
material needed to execute PanomiR.

## 1. Pathway summarization

PanomiR can generate pathway activity profiles given a gene expression
dataset and a list of pathways.Pathway summaries are numbers that
represent the overall activity of genes that belong to each pathway.
These numbers are calculated based on a methodology previously described
in part in (Altschuler et al. 2013; Joachim et al. 2018). Briefly, genes
in each sample are ranked by their expression values and then pathway
summaries are calculated as the average rank-squared of genes within a
pathway. The summaries are then center and scaled (zNormalized) across
samples.

The default list of background pathways in PanomiR is formatted into a
table (`data("path_gene_table")`). The table is based on canonical
pathways collection of Molecular Signatures Database (MSigDB) V6.2 and
it contains annotated pathways from a variety of sources (Liberzon et
al. 2011).

\*\* Users interested in using other pathway/geneset backgrounds, such
as newer versions of MSigDB or KEGG, should refer to the
[appendix](#geneset) of this manual.

This section uses a reduced example dataset from The Cancer Genome Atlas
(TCGA) Liver Hepatocellular Carcinoma (LIHC) dataset to generate Pathway
summary statistics (Ally et al. 2017). **Note:** Make sure that you
select gene representation type that matches the rownames of your
expression data. The type can be modified using the `id` argument in the
function below. The default value for this argument is `ENSEMBL`.

``` r
library(PanomiR)

# Pathway reference from the PanomiR package
data("path_gene_table")
data("miniTestsPanomiR")
# Generating pathway summary statistics 

summaries <- pathwaySummary(miniTestsPanomiR$mini_LIHC_Exp,
                            path_gene_table, method = "x2",
                            zNormalize = TRUE, id = "ENSEMBL")

head(summaries)[,1:2]
```

    ##                         TCGA-BC-A10S-01A-22R-A131-07
    ## BIOCARTA_41BB_PATHWAY                     -0.1506216
    ## BIOCARTA_ACE2_PATHWAY                     -0.5676447
    ## BIOCARTA_ACH_PATHWAY                      -0.3211747
    ## BIOCARTA_ACTINY_PATHWAY                    1.4363526
    ## BIOCARTA_AGPCR_PATHWAY                    -0.1948523
    ## BIOCARTA_AGR_PATHWAY                       0.6802993
    ##                         TCGA-BC-4073-01B-02R-A131-07
    ## BIOCARTA_41BB_PATHWAY                     -0.1269436
    ## BIOCARTA_ACE2_PATHWAY                     -0.8327436
    ## BIOCARTA_ACH_PATHWAY                      -0.4390042
    ## BIOCARTA_ACTINY_PATHWAY                    1.4975456
    ## BIOCARTA_AGPCR_PATHWAY                    -0.2499193
    ## BIOCARTA_AGR_PATHWAY                       0.5420588

## 2. Differential Pathway activation

Once you generate the pathway activity profiles, as discussed in the
last section, there are several analysis that you can perform. We have
bundled some of the most important ones into standalone functions. Here,
we describe differential pathway activation profiling, which is
examining differences in pathway activity profiles in user-determined
conditions.

At this stage you need to provide a pathway-gene association table, an
expression dataset, and a covariates table. You need to specity what
covariates you would like to contrast. You also need to provide a
contrast, as formatted in limma. If the contrast is not provided, the
function assumes the first two levels of the provided contrast
covariate. **Note:** make sure the contrast covariate is formatted as
factor.

``` r
output0 <- differentialPathwayAnalysis(
                        geneCounts = miniTestsPanomiR$mini_LIHC_Exp,
                        pathways =  path_gene_table,
                        covariates = miniTestsPanomiR$mini_LIHC_Cov,
                        condition = 'shortLetterCode')

de.paths <- output0$DEP

head(de.paths,3)
```

    ##                                                 logFC   AveExpr          t
    ## REACTOME_GROWTH_HORMONE_RECEPTOR_SIGNALING -0.9159376 0.3044281 -10.404966
    ## BIOCARTA_AKT_PATHWAY                       -0.5744103 0.3123897  -6.770069
    ## PID_IL5_PATHWAY                            -0.6219876 0.4240432  -6.255756
    ##                                                 P.Value   adj.P.Val        B
    ## REACTOME_GROWTH_HORMONE_RECEPTOR_SIGNALING 1.942463e-06 0.002012391 5.240095
    ## BIOCARTA_AKT_PATHWAY                       6.903010e-05 0.035757593 2.126311
    ## PID_IL5_PATHWAY                            1.276971e-04 0.040289104 1.550780
    ##                                                                       contrast
    ## REACTOME_GROWTH_HORMONE_RECEPTOR_SIGNALING shortLetterCodeTP-shortLetterCodeNT
    ## BIOCARTA_AKT_PATHWAY                       shortLetterCodeTP-shortLetterCodeNT
    ## PID_IL5_PATHWAY                            shortLetterCodeTP-shortLetterCodeNT

## 3. Finding clusters of pathways

PanomiR provides a function to find groups coordinated differentially
activated pathways based on a pathway co-expression network (PCxN)
previously described in (Pita-Juárez et al. 2018). Briefly, PCxN is a
network where nodes are pathways and links are co-expression between the
nodes. It is formatted into a table were rows represent edges. The edges
of PCxN are marked by two numbers, 1- a correlation co-efficient and 2-
a significance adjusted p-value. Cut-offs for both of these numbers can
be manually set using PanomiR functions. See function manuals for more
info.

PCxN and its associated genesets are already released and can be
accessed through following Bioconductor packages:
[pcxn](http://bioconductor.org/packages/release/bioc/html/pcxn.html) and
[pcxnData](http://bioconductor.org/packages/release/data/experiment/html/pcxnData.html).

Here we have provided a small version of PCxN for tutorial purposes. A
more recent version of PCxN based on MSigDB V6.2 is available through
the data repository accompanying PanomiR manuscript, which can be found
[here](https://github.com/pouryany/PanomiR_paper).

``` r
# using an updated version of pcxn 

set.seed(2)
pathwayClustsLIHC <- mappingPathwaysClusters(
                            pcxn = miniTestsPanomiR$miniPCXN, 
                            dePathways = de.paths[1:300,],
                            topPathways = 200,
                            outDir=".",
                            plot = FALSE,
                            subplot = FALSE,
                            prefix='',
                            clusteringFunction = "cluster_louvain",
                            correlationCutOff = 0.1)


head(pathwayClustsLIHC$Clustering)
```

    ##                      Pathway cluster
    ## 1       BIOCARTA_NO1_PATHWAY       1
    ## 2       BIOCARTA_AKT_PATHWAY       1
    ## 3       BIOCARTA_ALK_PATHWAY       1
    ## 4     BIOCARTA_RANKL_PATHWAY       1
    ## 5       BIOCARTA_MCM_PATHWAY       3
    ## 6 BIOCARTA_CELLCYCLE_PATHWAY       3

## 4. Prioritizing miRNAs per cluster of pathways.

PanomiR identifies miRNAs that target clusters of pathways, as defined
in the last section. In order to this, you would need a reference table
of miRNA-Pathway association score (enrichment). We recommend using a
customized miRNA-Pathway association table, tailored to your
experimental data. This section provides an overview of prioritization
process. Readers interested in knowing more about the technical details
of PanomiR are refered to accompaniying publication (Work under
preparation).

### Enrichment reference

Here, we provide a preprocessed small example table of miRNA-pathway
enrichment in `miniTestsPanomiR$miniEnrich` object. This table contains
enrichment analysis results using Fisher’s Exact Test between MSigDB
pathways and TargetScan miRNA targets. The individual components are
accessible via `data(msigdb_c2)` and `data(targetScan_03)` (Agarwal et
al. 2015; Liberzon et al. 2011). This example table is contains only a
full subset of the full pairwise enrichment. You can refer to [section
5](#geneset) of this manual on how to create full tables and how to
customize them to your specific gene expression data.

### Generating targeting scores

PanomiR generates a score for individual miRNAs targeting a group of
pathways. These scores are generated based on the reference enrichment
table. We are interested in knowing to what extent each miRNA targets
pathway clusters identified in the last step (see previous section).
PanomiR constructs a null distribution of this targeting score for each
miRNA. The significance of observed scores from a given group of
pathways (clusters in this case) is contrasted against the null
distribution to generate a targeting p-value. These p-values are used to
rank miRNAs per cluster.

### Sampling parameter

The above described process requires repeated sampling to empirically
obtain the null distribution. The argument `sampRate` denotes the number
of repeats in the process. Note that in the example below, we use a
sampling rate of 50, the recommended rate is between 500-1000. Also, we
set the saveSampling argument to FALSE. This argument, if set TRUE,
ensures that the null distribution is obtain only once. This argument
should be set to TRUE if you wish to save your sampling and check for
different outputs from the clustering algorithms or pathway thresholds.

``` r
set.seed(1)
output2 <- prioritizeMicroRNA(enriches0 = miniTestsPanomiR$miniEnrich,
                    pathClust = miniTestsPanomiR$miniPathClusts$Clustering,
                    topClust = 1,
                    sampRate = 50, 
                    method = c("aggInv"),
                    outDir = "Output/",
                    dataDir = "outData/",
                    saveSampling = FALSE,
                    runJackKnife = FALSE,
                    numCores = 1,
                    prefix = "outmiR",
                    saveCSV = FALSE)
```

    ## Working on Cluster1.

    ## Performing aggInv function.

    ## aggInv Method Done

``` r
head(output2$Cluster1)
```

    ##                                 x cluster_hits aggInv_cover  aggInv_pval
    ## 1                hsa-miR-101-3p.2            6   -1.9566603 0.0001216703
    ## 2                hsa-miR-101-3p.1            4   -0.3395771 0.0006214715
    ## 3 hsa-miR-124-3p.2/hsa-miR-506-3p            7   -0.2357761 0.0008599272
    ## 4                 hsa-miR-1247-5p            4   -1.6599230 0.0021625662
    ## 5                 hsa-miR-1249-3p            1   -2.4578993 0.0042061415
    ## 6                 hsa-miR-1252-5p            4   -0.7572036 0.0050836835
    ##    aggInv_fdr
    ## 1 0.002433406
    ## 2 0.005732848
    ## 3 0.005732848
    ## 4 0.010812831
    ## 5 0.016824566
    ## 6 0.016945612

## 5. miRNA-Pathway enrichment tables

PanomiR best performs on tissue/experiment-customized datasets. In order
to do this, you need to create a customized enrichment table. You can
simply do so by using the pathway and miRNA list that we have provided
as a part of the package. simply, plug in the name of the genes present
(expressed) in your experiment in the following code

``` r
# using an updated version of pcxn 
data("msigdb_c2")
data("targetScan_03")


customeTableEnrich <- miRNAPathwayEnrichment(mirSets = targetScan_03,
                                              pathwaySets = msigdb_c2,
                                              geneSelection = yourGenes,
                                              mirSelection = yourMicroRNAs,
                                              fromID = "ENSEMBL",
                                              toID = "ENTREZID",
                                              minPathSize = 9,
                                              numCores = 1,
                                              outDir = ".",
                                              saveOutName = NULL)
```

In the above section, the field `fromID` denotes the gene representation
format of your input list. Here is a quick example that runs fast. Note
that the `miRNAPathwayEnrichment()` function creates a detailed report
with parameters that are used internally. To get a smaller table that is
suitable for publication purposes, use `reportEnrichment()` function.

``` r
# using an updated version of pcxn 
data("msigdb_c2")
data("targetScan_03")

tempEnrich <-miRNAPathwayEnrichment(targetScan_03[1:30],msigdb_c2[1:30])

head(reportEnrichment(tempEnrich))
```

    ##                               miRNA                Pathway    pval pAdjust
    ## 14                  hsa-miR-1252-5p   BIOCARTA_CSK_PATHWAY 0.00179   0.259
    ## 55    hsa-miR-1271-5p/hsa-miR-96-5p   BIOCARTA_AKT_PATHWAY 0.01100   1.000
    ## 122                hsa-miR-124-3p.1 BIOCARTA_RANKL_PATHWAY 0.01550   1.000
    ## 53  hsa-miR-124-3p.2/hsa-miR-506-3p   BIOCARTA_AKT_PATHWAY 0.03740   1.000
    ## 99                  hsa-miR-1252-5p BIOCARTA_AGPCR_PATHWAY 0.03940   1.000
    ## 112                hsa-miR-124-3p.1   BIOCARTA_BCR_PATHWAY 0.05360   1.000

## 6. Customized genesets and recommendations

PanomiR can integrate genesets and pathways from external sources
including those annotated in MSigDB. In order to do so, you need to
provide a `GeneSetCollection` object as defined in the `GSEABase`
package.

The example below illustrates how to use external sources to create your
own customized pathway-gene association table. This customized can then
replaced `path_gene_table` input in functions described in sections 1,2,
and 5 of this manual.

``` r
data("gscExample")

newPathGeneTable <-tableFromGSC(gscExample)
```

    ## 

    ## 

    ## 'select()' returned 1:1 mapping between keys and columns
    ## 'select()' returned 1:1 mapping between keys and columns

``` r
head(newPathGeneTable)
```

    ## # A tibble: 6 × 3
    ##   Pathway                               ENTREZID ENSEMBL        
    ##   <chr>                                 <chr>    <chr>          
    ## 1 NIKOLSKY_BREAST_CANCER_16Q24_AMPLICON 124245   ENSG00000158545
    ## 2 NIKOLSKY_BREAST_CANCER_16Q24_AMPLICON 1800     ENSG00000015413
    ## 3 NIKOLSKY_BREAST_CANCER_16Q24_AMPLICON 404550   ENSG00000154102
    ## 4 NIKOLSKY_BREAST_CANCER_16Q24_AMPLICON 6687     ENSG00000197912
    ## 5 NIKOLSKY_BREAST_CANCER_16Q24_AMPLICON 750      ENSG00000221819
    ## 6 NIKOLSKY_BREAST_CANCER_16Q24_AMPLICON 2588     ENSG00000141012

The the pathway correlation network in section 3 is build upon an MSigDB
V6.2, canonical pathways (cp) collection dataset that includes KEGG
Pathways. KEGG prohibits distribution of its pathways by third parties.
However, use can access desired versions of MSigDB in gmt format via
[this link](https://www.gsea-msigdb.org/gsea/downloads_archive.jsp)
(Subramanian et al. 2005).

The library `msigdb` provides an programmatic interface to download
different geneset collections. Including how to add KEGG pathways or
download mouse genesets. Use the this [MSigDB
tutorial](https://bioconductor.org/packages/release/data/experiment/vignettes/msigdb/inst/doc/msigdb.html)
to create your desired gene sets.

You can also use the following code chunk to create pathway-gene
association tables from gmt files.

``` r
library(GSEABase)

yourGeneSetCollection <- getGmt("YOUR GMT FILE")
newPathGeneTable      <- tableFromGSC(yourGeneSetCollection)
```

## Session info

``` r
sessionInfo()
```

    ## R Under development (unstable) (2021-12-03 r81290)
    ## Platform: x86_64-apple-darwin17.0 (64-bit)
    ## Running under: macOS Big Sur/Monterey 10.16
    ## 
    ## Matrix products: default
    ## BLAS:   /Library/Frameworks/R.framework/Versions/4.2/Resources/lib/libRblas.0.dylib
    ## LAPACK: /Library/Frameworks/R.framework/Versions/4.2/Resources/lib/libRlapack.dylib
    ## 
    ## locale:
    ## [1] en_US.UTF-8/en_US.UTF-8/en_US.UTF-8/C/en_US.UTF-8/en_US.UTF-8
    ## 
    ## attached base packages:
    ## [1] stats     graphics  grDevices utils     datasets  methods   base     
    ## 
    ## other attached packages:
    ## [1] PanomiR_0.99.4
    ## 
    ## loaded via a namespace (and not attached):
    ##   [1] fgsea_1.21.0           colorspace_2.0-2       ggtree_3.3.1          
    ##   [4] ellipsis_0.3.2         qvalue_2.27.0          XVector_0.35.0        
    ##   [7] aplot_0.1.1            rstudioapi_0.13        farver_2.1.0          
    ##  [10] graphlayouts_0.8.0     ggrepel_0.9.1          bit64_4.0.5           
    ##  [13] scatterpie_0.1.7       AnnotationDbi_1.57.1   fansi_0.5.0           
    ##  [16] splines_4.2.0          cachem_1.0.6           GOSemSim_2.21.0       
    ##  [19] knitr_1.37             polyclip_1.10-0        jsonlite_1.7.2        
    ##  [22] annotate_1.73.0        GO.db_3.14.0           png_0.1-7             
    ##  [25] graph_1.73.0           ggforce_0.3.3          compiler_4.2.0        
    ##  [28] httr_1.4.2             assertthat_0.2.1       Matrix_1.4-0          
    ##  [31] fastmap_1.1.0          lazyeval_0.2.2         cli_3.1.0             
    ##  [34] limma_3.51.2           tweenr_1.0.2           htmltools_0.5.2       
    ##  [37] tools_4.2.0            igraph_1.2.11          gtable_0.3.0          
    ##  [40] glue_1.6.0             GenomeInfoDbData_1.2.7 reshape2_1.4.4        
    ##  [43] DO.db_2.9              dplyr_1.0.7            fastmatch_1.1-3       
    ##  [46] Rcpp_1.0.7             enrichplot_1.15.2      Biobase_2.55.0        
    ##  [49] vctrs_0.3.8            Biostrings_2.63.1      ape_5.6-1             
    ##  [52] nlme_3.1-153           ggraph_2.0.5           xfun_0.29             
    ##  [55] stringr_1.4.0          lifecycle_1.0.1        clusterProfiler_4.3.1 
    ##  [58] XML_3.99-0.8           DOSE_3.21.2            org.Hs.eg.db_3.14.0   
    ##  [61] zlibbioc_1.41.0        MASS_7.3-54            scales_1.1.1          
    ##  [64] tidygraph_1.2.0        parallel_4.2.0         RColorBrewer_1.1-2    
    ##  [67] yaml_2.2.1             memoise_2.0.1          gridExtra_2.3         
    ##  [70] ggplot2_3.3.5          downloader_0.4         ggfun_0.0.4           
    ##  [73] yulab.utils_0.0.4      stringi_1.7.6          RSQLite_2.2.9         
    ##  [76] S4Vectors_0.33.9       tidytree_0.3.6         BiocGenerics_0.41.2   
    ##  [79] BiocParallel_1.29.10   GenomeInfoDb_1.31.1    rlang_0.4.12          
    ##  [82] pkgconfig_2.0.3        bitops_1.0-7           evaluate_0.14         
    ##  [85] lattice_0.20-45        purrr_0.3.4            treeio_1.19.1         
    ##  [88] patchwork_1.1.1        shadowtext_0.1.0       bit_4.0.4             
    ##  [91] tidyselect_1.1.1       GSEABase_1.57.0        plyr_1.8.6            
    ##  [94] magrittr_2.0.1         R6_2.5.1               IRanges_2.29.1        
    ##  [97] generics_0.1.1         DBI_1.1.2              pillar_1.6.4          
    ## [100] withr_2.4.3            KEGGREST_1.35.0        RCurl_1.98-1.5        
    ## [103] tibble_3.1.6           crayon_1.4.2           utf8_1.2.2            
    ## [106] rmarkdown_2.11         viridis_0.6.2          grid_4.2.0            
    ## [109] data.table_1.14.2      blob_1.2.2             forcats_0.5.1         
    ## [112] digest_0.6.29          xtable_1.8-4           tidyr_1.1.4           
    ## [115] gridGraphics_0.5-1     stats4_4.2.0           munsell_0.5.0         
    ## [118] viridisLite_0.4.0      ggplotify_0.1.0

## References

<div id="refs" class="references csl-bib-body hanging-indent">

<div id="ref-agarwal2015predicting" class="csl-entry">

Agarwal, Vikram, George W Bell, Jin-Wu Nam, and David P Bartel. 2015.
“Predicting Effective microRNA Target Sites in Mammalian mRNAs.” *Elife*
4: e05005.

</div>

<div id="ref-ally2017comprehensive" class="csl-entry">

Ally, Adrian, Miruna Balasundaram, Rebecca Carlsen, Eric Chuah, Amanda
Clarke, Noreen Dhalla, Robert A Holt, et al. 2017. “Comprehensive and
Integrative Genomic Characterization of Hepatocellular Carcinoma.”
*Cell* 169 (7): 1327–41.

</div>

<div id="ref-altschuler2013pathprinting" class="csl-entry">

Altschuler, Gabriel M, Oliver Hofmann, Irina Kalatskaya, Rebecca Payne,
Shannan J Ho Sui, Uma Saxena, Andrei V Krivtsov, et al. 2013.
“Pathprinting: An Integrative Approach to Understand the Functional
Basis of Disease.” *Genome Medicine* 5 (7): 1–13.

</div>

<div id="ref-joachim2018relative" class="csl-entry">

Joachim, Rose B, Gabriel M Altschuler, John N Hutchinson, Hector R Wong,
Winston A Hide, and Lester Kobzik. 2018. “The Relative Resistance of
Children to Sepsis Mortality: From Pathways to Drug Candidates.”
*Molecular Systems Biology* 14 (5): e7998.

</div>

<div id="ref-liberzon2011molecular" class="csl-entry">

Liberzon, Arthur, Aravind Subramanian, Reid Pinchback, Helga
Thorvaldsdóttir, Pablo Tamayo, and Jill P Mesirov. 2011. “Molecular
Signatures Database (MSigDB) 3.0.” *Bioinformatics* 27 (12): 1739–40.

</div>

<div id="ref-pita2018pathway" class="csl-entry">

Pita-Juárez, Yered, Gabriel Altschuler, Sokratis Kariotis, Wenbin Wei,
Katjuša Koler, Claire Green, Rudolph E Tanzi, and Winston Hide. 2018.
“The Pathway Coexpression Network: Revealing Pathway Relationships.”
*PLoS Computational Biology* 14 (3): e1006042.

</div>

<div id="ref-subramanian2005gene" class="csl-entry">

Subramanian, Aravind, Pablo Tamayo, Vamsi K Mootha, Sayan Mukherjee,
Benjamin L Ebert, Michael A Gillette, Amanda Paulovich, et al. 2005.
“Gene Set Enrichment Analysis: A Knowledge-Based Approach for
Interpreting Genome-Wide Expression Profiles.” *Proceedings of the
National Academy of Sciences* 102 (43): 15545–50.

</div>

</div>

#' Outputs a table with pathways and their respective clusters
#'
#' @param pcxn pathways network (edge list of pathways)
#' @param dePathways differential expressed pathways, obtained from
#'   *DifferentialPathwayAnalysis*
#' @param clusteringFunction clustering algorithm
#' @param edgeFDR FDR threshold for pathway-pathway adjusted p-values;
#'   filter edges with adjusted p-values less than given threshold
#' @param correlationCutOff cut-off threshold for pathway-pathway correlation;
#'   filter pathways with correlation less than given threshold
#' @param pathwayFDR FDR threshold for DE pathways adjusted p-values;
#'   filter pathways with adjusted p-values less than given threshold
#' @param topPathways  use only top x paths; if NULL, use all paths
#' @param plotOut if TRUE, store graph plot in Figures directory of plots
#' @param subplot if TRUE, store individual clusters plots and connected plots
#'   in Figures directory of plots
#' @param topClusters plot figures for top x clusters
#' @param outDir output directory
#' @param saveNameCSV if not NULL, saves output as csv using save name
#' @param prefix add prefix to plots
#' @param weighted True if you wish to include correlation weights in clustering
#' @return a list where the first item is a table with each row containing
#'   a pathway and its respective cluster. The second item is an igraph object.
#' @examples
#' data("miniTestsPanomiR")
#'
#' mappingPathwaysClusters(pcxn = miniTestsPanomiR$miniPCXN,
#'                          dePathways = miniTestsPanomiR$miniDEP,
#'                          topPathways = 200,
#'                          outDir=".",
#'                          plot = FALSE,
#'                          subplot = FALSE,
#'                          prefix='',
#'                          clusteringFunction = "cluster_louvain",
#'                          correlationCutOff = 0.1)
#'
#' @export
mappingPathwaysClusters <- function(pcxn, dePathways, clusteringFunction = NULL,
                            edgeFDR = 0.05, correlationCutOff = 0.316,
                            pathwayFDR = 0.05, topPathways = 200,
                            plotOut = TRUE, subplot = TRUE, topClusters = 2,
                            prefix = "", outDir = ".", saveNameCSV = NULL,
                            weighted = FALSE) {
    outDir <- paste0(outDir, "/")
    if (!dir.exists(outDir)) stop("Output directory does not exist.")
    figDir <- paste0(outDir, "Figures/")
    if (!dir.exists(figDir) && plotOut) dir.create(figDir, recursive = TRUE)

    tempNet     <- pcxnToNet(pcxn, edgeFDR, correlationCutOff, weighted)
    net         <- tempNet$net
    allPathways <- tempNet$allPathways
    dePathways <- dePathways[dePathways$adj.P.Val < pathwayFDR, ]
    if (nrow(dePathways) < 10) stop("Relax pathway adjusted p-values.")

    if (!is.null(topPathways)) {
        dePathways <- dePathways[seq_len(topPathways), ]
    }
    dePathways1 <- dePathways
    dePathways  <- dePathways[which(rownames(dePathways) %in% allPathways), ]

    igraph::V(net)$shape <- ifelse(igraph::V(net)$name %in% dePathways,
                                "square", "circle")
    subNet <- igraph::induced_subgraph(net, rownames(dePathways))
    # choose clustering function for nodes
    if (is.null(clusteringFunction)) {
        clusts <- igraph::cluster_edge_betweenness(subNet)
    } else {
        clusts <- utils::getFromNamespace(clusteringFunction, "igraph")(subNet)
    }
    clusts <- .levelFixerMapper(clusts)
    subNet <- .shapeColNet(subNet, dePathways1)
    igraph::V(subNet)$cluster <- clusts$membership
    pathsOut <- igraph::V(subNet)$name
    pathsOut <- as.data.frame(cbind("Pathway" = pathsOut,
                                    "cluster" = clusts$membership))
    if (plotOut == TRUE) {
        clusterPlot(subNet = subNet, subplot = subplot,
                topClusters = topClusters, outDir = figDir, prefix = prefix)
    }
    if (!is.null(saveNameCSV)) {
        utils::write.csv(pathsOut, paste0(outDir, saveNameCSV))
    }
    return(list("Clustering" = pathsOut, "DE_PCXN" = subNet,
                "Cluter_method" = eval(clusteringFunction)))
}

utils::globalVariables(c(
    ".", ":=", "ENSEMBL", "ES", "ES2", "Intersect", "Pathway", "hit",
    "hit2", "k", "n", "path_fdr", "pval", "x", "y"
))

#' Pathway-Gene Associations
#'
#' Generates a table of pathways and genes associations.
#'
#' @param pathAdress Address to an RDS file containing list of pathways where
#'   each element is a list of genes similar to GMT format.
#' @param pathwayList If you wish to use a list of pathways instead of a file
#'   use this argument instead. The list must contain no NA values.
#' @param fromType gene annotation type used in your input data.
#' @param toType gene annotation type to be produced in the output.
#' @param outDir Address to save an RDS for a table of pathway-gene association
#' @return pathExpTab Table of pathway-gene association.
#' @examples
#' pathway1 <- c("125", "3099", "126")
#' pathway2 <- c("5232", "5230", "5162")
#' pathList <- list("Path1" = pathway1, "Path2" = pathway2)
#' res <- pathwayGeneTab(pathwayList = pathList)
#'
#' data(msigdb_c2)
#' pathwayGeneTab(pathwayList = msigdb_c2[1:2])
#' @export
pathwayGeneTab <- function(pathAdress = NA, pathwayList = NA,
                    fromType = "ENTREZID", toType = "ENSEMBL", outDir = NA) {
    if (!is.na(pathAdress)) {
        pathList <- readRDS(pathAdress)
    }

    if (is.na(pathAdress) && !any(is.na(pathwayList))) {
        pathList <- pathwayList
    }

    if (!is.na(pathAdress) && !any(is.na(pathwayList))) {
        stop("provide a valid input list.")
    }

    pathList2 <- lapply(
        pathList,
        function(x) {
            clusterProfiler::bitr(x, fromType = fromType, toType = toType,
                OrgDb = org.Hs.eg.db::org.Hs.eg.db)
        }
    )

    temp <- lapply(
        names(pathList2),
        function(x) {
            data.frame(
                Pathway = (x),
                pathList2[[x]]
            )
        }
    )
    temp <- do.call(rbind, (temp))
    pathExpTab <- tibble::as_tibble(temp)

    if (!is.na(outDir)) {
        saveRDS(pathExpTab, outDir)
    }

    return(pathExpTab)
}

#' Pathway-Gene Associations from GeneSet collections
#'
#' This function enables to utilize MSigDB packages and GSEABase objects
#'   to incorporate customized genesets into PanomiR.
#'
#' @param gsCollection An GSEABase gene set collection object
#' @param fromType gene annotation type used in your input data
#' @param toType gene annotation type to be produced in the output
#' @return A table of pathway-gene associations
#' @examples
#' data(gscExample)
#' tableFromGSC(gscExample)
#' @export
tableFromGSC <- function(gsCollection, fromType = "ENTREZID",
                                toType = "ENSEMBL") {
    gsList <- GSEABase::geneIds(gsCollection)
    pathwayGeneTab(pathwayList = gsList, fromType = fromType, toType = toType)
}

#' Pathway Summary Statistics
#'
#' Generates a table of pathway activity profiles per sample
#'
#' @param exprsMat Gene expression matrix with row names as genes and samples
#'   as columns.
#' @param pathwayRef Table of pathway-gene associations. Created from
#'   \code{\link{pathwayGeneTab}} function.
#' @param id Gene annotation type in the row name of gene expression data.
#' @param zNormalize Normalization of pathway summary score.
#' @param method Choice of how to summarize gene ranks into pathway statistics.
#' @param deGenes List of differentially expressed genes along with t-scores.
#'   Only necessary if working on Top 50\% summary method.
#' @param trim Percentage of top and bottom ranked genes to be excluded from
#'   pathway summary statistics.
#' @param tScores Argument for-top-50-percent-genes method.
#' @return pathExp Table of pathway activity profiles per sample.
#' @examples
#' pathTab <- tibble::tribble(
#'  ~Pathway, ~ENTREZID,  ~ENSEMBL,
#'  "Path1",   "125",      "ENSG00000196616",
#'  "Path1",   "3099",     "ENSG00000159399",
#'  "Path2",   "5230",     "ENSG00000102144",
#'  "Path2",   "5162",     "ENSG00000168291"
#'  )
#' exprsMat <- matrix(2 * (seq_len(12)), 4, 3)
#' rownames(exprsMat) <- pathTab$ENSEMBL
#' colnames(exprsMat) <- LETTERS[seq_len(3)]
#' pathwaySummary(exprsMat, pathTab, method = "x2")
#' @export

pathwaySummary <- function(exprsMat, pathwayRef, id = "ENSEMBL",
                    zNormalize = FALSE, method = FALSE, deGenes = NULL,
                    trim = 0, tScores = NULL) {
    # Current version only works with ENSEMBL.
    exprsMat <- tibble::rownames_to_column(as.data.frame(exprsMat), var = id)
    if (!is.null(deGenes)) {
        if (is.null(tScores)) {
            stop("Provide tscores/pvalues")
        }
        pathwayRef <- dplyr::inner_join(pathwayRef, tScores, by = c("ENSEMBL"))
        pathwayRef <- pathwayRef |> dplyr::group_by(Pathway) |>
            dplyr::filter(abs(t) >= stats::median(abs(t))) |>
            dplyr::select(-t)
    }
    if (method == "none") {
        exprsMat <- exprsMat |>
            dplyr::mutate_if(is.numeric, function(x) {
                x
                })
    } else if (method == "x") {
        exprsMat <- exprsMat |>
            dplyr::mutate_if(is.numeric, rank) |>
            dplyr::mutate_if(is.numeric, function(x) {
                x
                })
    } else if (method == "x2") {
        exprsMat <- exprsMat |>
            dplyr::mutate_if(is.numeric, rank) |>
            dplyr::mutate_if(is.numeric, function(x) {
                x * x
            })
    } else {
        stop("invalid choice of summarization function")
    }
    pathExpTab <- dplyr::inner_join(pathwayRef, exprsMat, by = id)
    pathExpTab <- pathExpTab |>
        dplyr::group_by(Pathway) |>
        dplyr::summarise_if(is.numeric, mean, na.rm = TRUE, trim = trim)

    pathExp <- as.data.frame(pathExpTab[, -1])
    rownames(pathExp) <- as.character(dplyr::pull(pathExpTab[, 1]))

    if (zNormalize) {
        pathExp <- apply(pathExp, 2, function(x) {
            (x - mean(x)) / stats::sd(x)
        })
    }
    return(pathExp)
}


### ==========================================================
#-----
#-----
#----- The following are miRNA-Pathway pval aggregations
#-----
#-----
### ==========================================================



#' Score miRNAs In a Cluster Of Pathways
#'
#' The function to count the number of enriched pathways for each miRNA.
#'
#' @param enriches Table of miRNA pathway enrichments.
#' @param pathways Queried pathways, e.g. cluster pathways.
#' @param isSelector Internal argument.
#' @param thresh Threshold from p-value cut-off.
#' @return P-value based scoring of miRNAs in a cluster of pathways.
#' @keywords internal
pCutFn <- function(enriches, pathways, isSelector, thresh = 0.05) {
    if (isSelector == TRUE) {
        enriches <- enriches |>
            dplyr::mutate(hit2 = ifelse(pval < thresh, 1, 0))
    }

    tempEnrich <- enriches[enriches$y %in% pathways, ]
    selector <- tempEnrich |>
        dplyr::group_by(x) |>
        dplyr::summarise(n = dplyr::n(), k = sum(hit2)) |>
        dplyr::arrange(x)

    if (isSelector == TRUE) {
        selector <- selector |> dplyr::filter(k > thresh * length(pathways))
        return(list("selector" = selector, "enriches0" = enriches))
    } else {
        return(selector)
    }
}


#' The function calculate targeting score of miRNA w.r.t to a cluster
#' of pathways via inverse normal method
#' @param enriches a table of miRNA pathway enrichments. Universe
#' @param pathways queried pathways. e.g. cluster pathways
#' @param isSelector internal argument
#' @param thresh internal argument
#' @return a  scoring of miRNAs in a cluster of pathways
#' @keywords internal
aggInvFn <- function(enriches, pathways, isSelector = TRUE, thresh = NULL) {
    if (isSelector == TRUE) {
        enriches <- enriches |> dplyr::mutate(ES2 = stats::qnorm(1 - pval))
        minES <- min(enriches$ES2[!is.infinite(enriches$ES2)])
        enriches <- enriches |>
            dplyr::mutate(ES2 = ifelse(is.infinite(enriches$ES2), minES,
                enriches$ES2))
    }

    tempEnrich <- enriches[enriches$y %in% pathways, ]
    selector <- tempEnrich |>
        dplyr::group_by(x) |>
        dplyr::summarise(n = dplyr::n(), k = mean(ES2)) |>
        dplyr::arrange(x)

    if (isSelector == TRUE) {
        return(list("selector" = selector, "enriches0" = enriches))
    } else {
        return(selector)
    }
}


#' The function calculate targeting score of miRNA w.r.t to a cluster
#' of pathways via log aggregation method.
#' @param enriches a table of miRNA pathway enrichments. Universe
#' @param pathways queried pathways. e.g. cluster pathways
#' @param isSelector internal argument
#' @param thresh internal argument
#' @return a  scoring of miRNAs in a cluster of pathways
#' @keywords internal
aggLogFn <- function(enriches, pathways, isSelector, thresh = 0) {
    enriches <- enriches |> dplyr::mutate(ES = -log(pval))
    if (isSelector == TRUE) {
        enriches <- enriches |> dplyr::mutate(ES = -log(pval))
    }

    tempEnrich <- enriches[enriches$y %in% pathways, ]
    selector <- tempEnrich |>
        dplyr::group_by(x) |>
        dplyr::summarise(n = dplyr::n(), k = mean(ES))

    if (isSelector == TRUE) {
        selector <- selector |>
            dplyr::filter(k * n >= thresh * length(pathways))
        return(list("selector" = selector, "enriches0" = enriches))
    } else {
        return(selector)
    }
}


#' The function calculate targeting score of miRNA w.r.t to a cluster
#' of pathways via sumz aggregation method.
#' @param enriches a table of miRNA pathway enrichments. Universe
#' @param pathways queried pathways. e.g. cluster pathways
#' @param isSelector internal argument
#' @param thresh internal argument
#' @return a  scoring of miRNAs in a cluster of pathways
#' @keywords internal
sumzFn <- function(enriches, pathways, isSelector, thresh = NULL) {
    enriches1 <- enriches |>
        dplyr::mutate(pval = ifelse(pval >= 0.999, 0.999, pval))

    enriches1 <- enriches1 |>
        dplyr::mutate(pval = ifelse(pval <= 1.0e-16, 1.0e-16, pval))

    tempEnrich <- enriches1[enriches1$y %in% pathways, ]
    aggPTab <- vector()
    for (i in unique(tempEnrich$x)) {
        temp <- tempEnrich[tempEnrich$x == i, ]
        tPVal <- metap::sumz(temp$pval)
        aggPTab <- rbind(aggPTab, c(i, tPVal$p, nrow(temp)))
    }

    selector <- tibble::tibble(
        "x" = aggPTab[, 1],
        "pval" = signif(as.numeric(aggPTab[, 2]), 4),
        "n" =  aggPTab[, 3]
    )

    if (isSelector == TRUE) {
        return(list("selector" = selector, "enriches0" = enriches))
    } else {
        return(selector)
    }
}


#' The function calculate targeting score of miRNA w.r.t to a cluster
#' of pathways via sumlog aggregation method.
#' @param enriches a table of miRNA pathway enrichments. Universe
#' @param pathways queried pathways. e.g. cluster pathways
#' @param isSelector internal argument
#' @param thresh internal argument
#' @return a  scoring of miRNAs in a cluster of pathways
#' @keywords internal
sumlogFn <- function(enriches, pathways, isSelector, thresh = NULL) {
    enriches1 <- enriches |> dplyr::mutate(pval = ifelse(pval >= 0.999,
                                    0.999, pval))
    enriches1 <- enriches1 |> dplyr::mutate(pval = ifelse(pval <= 1.0e-16,
                                    1.0e-16, pval))

    tempEnrich <- enriches1[enriches1$y %in% pathways, ]
    aggPTab <- vector()
    for (i in unique(tempEnrich$x)) {
        temp <- tempEnrich[tempEnrich$x == i, ]
        tPVal <- metap::sumlog(temp$pval)
        aggPTab <- rbind(aggPTab, c(i, tPVal$p, nrow(temp)))
    }

    selector <- tibble::tibble(
        "x" = aggPTab[, 1],
        "pval" = signif(as.numeric(aggPTab[, 2]), 4),
        "n" =  aggPTab[, 3]
    )

    if (isSelector == TRUE) {
        return(list("selector" = selector, "enriches0" = enriches))
    } else {
        return(selector)
    }
}


#' Internal function for modification of prioritization.
#' @param selector a prioritzation table
#' @param coverName a new column name
#' @return an updated scoring of miRNAs in a cluster of pathways
#' @keywords internal
pCutCoverFn <- function(selector, coverName) {
    selector <- selector |>
        dplyr::mutate(!!coverName := k / n)
    return(selector)
}


#' Internal function for modification of prioritization.
#' @param selector a prioritzation table
#' @param coverName a new column name
#' @return an updated scoring of miRNAs in a cluster of pathways
#' @keywords internal
aggInvCoverFn <- function(selector, coverName) {
    selector <- selector |>
        dplyr::mutate(!!coverName := k)
    return(selector)
}

#' Internal function for modification of prioritization.
#' @param selector a prioritzation table
#' @param coverName a new column name
#' @return an updated scoring of miRNAs in a cluster of pathways
#' @keywords internal
aggLogCoverFn <- aggInvCoverFn

#' Internal function for modification of prioritization.
#' @param selector a prioritzation table
#' @param coverName a new column name
#' @return an updated scoring of miRNAs in a cluster of pathways
#' @keywords internal
sumzCoverFn <- aggInvCoverFn

#' Internal function for modification of prioritization.
#' @param selector a prioritzation table
#' @param coverName a new column name
#' @return an updated scoring of miRNAs in a cluster of pathways
#' @keywords internal
sumlogCoverFn <- aggInvCoverFn


#### Working

#' Outputs a table of sampling data(rows are miRNA and cols are samples)
#'
#' @param enrichNull Enrichment dataset with x (miRNA), y (pathway) and pval
#'   (probability of observing x in pathway cluster).
#' @param selector Table with x(miRNA) in pathway cluster.
#' @param sampRate Sampling rate.
#' @param fn Methodology function.
#' @param nPaths Number of pathways in pathway cluster.
#' @param samplingDataFile If file exists, load. Else, perform random sampling
#' @param saveSampling If TRUE, data is saved.
#' @param jackKnife If TRUE, conduct sampling with one less pathway, used for
#'   jack knifing
#' @param numCores number of cores used
#' @param autoSeed random permutations are generated based on predetermined
#'   seeds. TRUE will give identical results in different runs.
#' @return Outputs of sampling data.
samplingDataBase <- function(enrichNull, selector, sampRate, fn, nPaths,
                        samplingDataFile, jackKnife = FALSE, saveSampling,
                        numCores = 1, autoSeed = TRUE) {
    if (!all(utils::hasName(selector, c("x")))) {
        stop("The selector table needs a column x (miRNA name)")
    }
    if (!all(utils::hasName(enrichNull, c("x", "y", "pval")))) {
        stop("Bad formatting in the enrichment table")
    }
    if (!file.exists(samplingDataFile)) {
        allPaths <- unique(enrichNull$y)
        if (jackKnife == TRUE) {
            nPathsTemp <- nPaths - 1
        }
        sampSizeVec <- c(nPaths, nPaths - 1, 100, 50)
        outList <- list()
        for (nPathsTemp in sampSizeVec) {
            temp <- parallel::mclapply(seq_len(sampRate), function(y) {
                if (autoSeed) {
                    withr::with_seed(y, {
                        nullPaths <- sample(allPaths, nPathsTemp,
                                        replace = FALSE)})
                }
                selNull <- fn(enriches = enrichNull, pathways = nullPaths,
                    isSelector = FALSE)
                return(selNull$k)
            }, mc.cores = numCores)
            # build null distribution of K
            temp <- do.call(rbind, temp)
            temp <- t(temp)
            rownames(temp) <- selector$x
            colnames(temp) <- vapply(seq_len(sampRate), function(y) {
                paste0("sample_", y)
            }, FUN.VALUE = "character")
            sampTag <- paste0("SampSize_", nPathsTemp)
            outList[[sampTag]] <- temp
        }
        if (saveSampling == TRUE) {
            saveRDS(outList, file = samplingDataFile)
            sayThis <- paste0(samplingDataFile, " saved.")
            message(sayThis)
        }
    } else {
        outList <- readRDS(samplingDataFile)
        sayThis <- paste0("Skipping sampling, ", samplingDataFile, " exists. ",
                        samplingDataFile, " loaded.")
        message(sayThis)
    }
    return(outList)
}

#' Outputs a table with col x, miRNA, probability of observing k
#'   against a random distribution of the cover of methodology
#'
#' @param samplingData Random distribution data.
#' @param selector Table with x(miRNA) in pathway cluster and observed
#'   k (depending on methodology).
#' @param m Method name.
#' @param nPaths Number of pathways used to generate the samplingData at
#'   each iteration. Default is set at 100.
#' @param coverFn Cover of methodology function.
#' @return Outputs a new selector table with col x, pval and cover.
methodProbBase <- function(samplingData,
                            selector,
                            m,
                            nPaths = 100,
                            coverFn = NULL) {
    if (!all(utils::hasName(selector, c("x", "k")))) {
        stop(
            "The selector needs a column x (miRNA) and a column k (miRNA hits)"
            )
    }

    # obtain means and sds for distribution, assume CLT
    means <- rowMeans(samplingData)
    sds <- apply(samplingData, 1, stats::sd)
    sds <- sds * 10 / sqrt(nPaths)

    pvalName <- paste0(m, "_pval")
    coverName <- paste0(m, "_cover")

    # obtain p-vals
    pVals <-
        stats::pnorm(selector$k, mean = means, sd = sds, lower.tail = FALSE)
    selector <- selector |>
        dplyr::mutate(!!pvalName := pVals) |>
        coverFn(coverName) |>
        dplyr::select(-c(k, n))
    return(selector)
}


#' Outputs a table with col x (miRNA), probability of observing
#' k (depending on methodology) against a random distribution with
#' jack-knifing of the pathway cluster (removing a pathway at a time)
#'
#' @param selector Table with x(miRNA) in pathway cluster and observed k
#'   (depending on methodology).
#' @param pathways Pathways in pathway cluster.
#' @param enrichNull Enrichment dataset with x (miRNA), y (pathway) and pval
#'   (probability of observing x in pathway cluster).
#' @param fn Methodology function.
#' @param jackKnifeData Random distribution data with jack-knifing
#'   (i.e. one less pathway)
#' @param m method name
#' @param numCores number of cores
#' @return Outputs a new selector table with col x, pval_jk
jackKnifeBase <- function(selector,
                            pathways,
                            enrichNull,
                            fn,
                            jackKnifeData,
                            m,
                            numCores = 1) {
    # obtain means and sds for distribution, assume CLT
    nPaths <- length(pathways)
    sampleMeans <- rowMeans(jackKnifeData)
    sampleSDs <- apply(jackKnifeData, 1, stats::sd)
    sampleSDs <- sampleSDs * 10 / sqrt(nPaths - 1)

    # remove one pathway at a time and obtain K for each miRNA
    temp1 <- parallel::mclapply(seq_along(pathways), function(x) {
        tempPathways <- pathways[-x]
        tempSelector <- fn(
            enriches = enrichNull, pathways = tempPathways,
            isSelector = FALSE
        )
        # obtain p-values using the means and sds obtain above
        pVals  <- stats::pnorm(tempSelector$k, mean = sampleMeans,
                        sd = sampleSDs, lower.tail = FALSE)

        return(pVals)
    }, mc.cores = numCores)

    # rows <- number of pathways; col <- number of miRNAs
    # contain p-values
    temp1 <- do.call(rbind, temp1)
    temp1 <- t(temp1)

    ## added from here
    jackKnifeName <- paste0(m, "_pval_jk")

    # obtain aggregate p-values
    means <- rowMeans(temp1)
    selector <- selector |>
        dplyr::mutate(!!jackKnifeName := means)

    return(selector)
}

#' Obtain Design Matrix
#'
#' Modified from covariates pipeline of Menachem Former. Imported from
#' \url{https://github.com/th1vairam/CovariateAnalysis}
#'
#' @param covariatesDataFrame Dataframe of covariates.
#' @param intercept intercept in the linear model.
#' @param reLevels TBA.
#' @return List containing a design matrix.
#' @examples
#' data(iris)
#' getDesignMatrix(iris)
#' @export
getDesignMatrix <- function(covariatesDataFrame, intercept = TRUE,
                            reLevels = list()) {
    covDF <- covariatesDataFrame
    rowNamesTemp <- rownames(covDF)
    colNamesTemp <- colnames(covDF)
    factorCovariateNames <- names(covDF)[vapply(covDF, is.factor, logical(1))]
    factorCovariateNames <- setdiff(factorCovariateNames,
        factorCovariateNames[!(factorCovariateNames %in% colnames(covDF))])

    numericCovariateNames <- setdiff(colNamesTemp, factorCovariateNames)

    # Ensure the factors are factor, and the quantitatives are numeric:
    covDF <- as.data.frame(lapply(colnames(covDF),
        function(column) {
            if (column %in% factorCovariateNames) {
                fac <- as.factor(covDF[, column])
                if (column %in% names(reLevels)) {
                    fac <- stats::relevel(fac, ref = reLevels[[column]])
                }
                return(fac)
            } else {
                return(as.numeric(covDF[, column]))
            }
        }))
    rownames(covDF) <- rowNamesTemp
    colnames(covDF) <- colNamesTemp

    tempHelper <- .getDesignMatHelper(factorCovariateNames, covDF, intercept)
    contra   <- tempHelper$contra
    covDF    <- tempHelper$covDF
    # Inspired by http://stackoverflow.com/questions/5616210/
    currentNAAction <- getOption("na.action")
    # Model matrix will now include "NA":
    options(na.action = "na.pass")

    if (intercept) {
        design <- stats::model.matrix(~., data = covDF, contrasts.arg = contra)
    } else {
        design <- stats::model.matrix(~ 0 + ., data = covDF,
                            contrasts.arg = contra)
    }

    rownames(design) <- rownames(covDF)
    options(na.action = currentNAAction)

    return(list(design = design, covariates = colNamesTemp,
        factorsLevels = lapply(contra, colnames),
        numericCovars = numericCovariateNames, covariatesDataFrame = covDF))
}

.getDesignMatHelper <- function(factorCovariateNames, covDF,
                                intercept) {
    contra <- NULL
    maxNumCat <- Inf
    catData <- covDF[, factorCovariateNames, drop = FALSE]
    if (ncol(catData) > 0) {
        numCats <- vapply(colnames(catData),
            function(col) nlevels(factor(catData[, col])), numeric(1))
        excludeCategoricalCols <-
            names(numCats)[numCats <= 1 | numCats > maxNumCat]

        if (!is.null(excludeCategoricalCols) &&
            length(excludeCategoricalCols) > 0) {
            warning(paste("Excluding categorical variables with less than 2",
                ifelse(is.infinite(maxNumCat), "",
                    paste(" or more than ", maxNumCat, sep = "")),
                " categories: ",
                paste(paste("'", excludeCategoricalCols,
                "'", sep = ""), collapse = ", "), sep = ""))

            factorCovariateNames <- setdiff(factorCovariateNames,
                excludeCategoricalCols)
            covDF <- covDF[, !(colnames(covDF) %in% excludeCategoricalCols),
                drop = FALSE]
        }
        # Inspired by http://stackoverflow.com/questions/4560459/
        # And, already ensured above that covDF[, factorCovariateNames] :
        # 1) fac is of type factor.
        # 2) fac is releveled as designated in reLevels.
        if (intercept) {
            contra <- lapply(
                factorCovariateNames,
                function(column) {
                    fac <- covDF[, column]
                    fac <- stats::contrasts(fac)
                }
            )
        } else {
            contra <- lapply(factorCovariateNames,
                function(column) {
                    fac <- covDF[, column]
                    fac <- stats::contrasts(fac, contrasts = FALSE)
                })
        }
        names(contra) <- factorCovariateNames
    }
    return <- list("contra" = contra, "covDF" = covDF)
}

#' Function imported from https://github.com/th1vairam/CovariateAnalysis
#' Modified from http://stackoverflow.com/questions/13088770/
#' Function to find linearly dependednt columns of a matrix
#' @param mat an input design matrix.
#' @return a list of independent columns
#' @examples
#' data("iris")
#' designMat <- getDesignMatrix(iris)
#' linColumnFinder(designMat$design)
#' @export
linColumnFinder <- function(mat) {
    mat[is.na(mat)] <- 0

    # If the matrix is full rank then we're done
    if (qr(mat)$rank == ncol(mat)) {
        return(list(
            indepCols = seq(1, ncol(mat), 1),
            relations = "Matrix is of full rank"
        ))
    }

    m <- ncol(mat)
    # cols keeps track of which columns are linearly independent
    cols <- 1
    allMessages <- c()
    for (i in seq(2, m)) {
        ids <- c(cols, i)
        mymat <- mat[, ids]
        if (qr(mymat)$rank != length(ids)) {
            # Regression the column of interest on the previous columns to
            # figure out the relationship
            o <- stats::lm(mat[, i] ~ as.matrix(mat[, cols]) + 0)
            # Construct the output message
            start <- paste0(colnames(mat)[i], " = ")
            # Which coefs are nonzero
            nz <- !(abs(stats::coef(o)) <= .Machine$double.eps^0.5)
            tmp <- colnames(mat)[cols[nz]]
            vals <- paste(stats::coef(o)[nz], tmp, sep = "*", collapse = " + ")
            amessage <- paste0(start, vals)
            allMessages <- c(allMessages, amessage)
        } else {
            # If the matrix subset was of full rank
            # then the newest column in linearly independent
            # so add it to the cols list
            cols <- ids
        }
    }
    return(list(indepCols = cols, relations = allMessages))
}

####
#
# Helper functions for differentialPathwayAnalysis
#
####



#' function to get residuals with respect to a set of covariates
#'
#' @param covariates a covariate dataframe.
#' @param adjustCovars covariates to adjust for
#' @param pathSumStats an expression matrix
#' @return a matrix of adjusted expression
getResidual <- function(covariates,
                        adjustCovars,
                        pathSumStats) {
    # Getting pathway residuals
    resDesingMat <- getDesignMatrix(covariates[, c(adjustCovars), drop = FALSE],
        intercept = FALSE)

    resDesingMat$design <- resDesingMat$design[,
        linColumnFinder(resDesingMat$design)$indepCols]

    fitResiduals <- limma::lmFit(pathSumStats, resDesingMat$design)
    fitResiduals <- limma::residuals.MArrayLM(fitResiduals, pathSumStats)

    return(fitResiduals)
}

#' function to get a DE table
#'
#' @param expMat an expression matrix
#' @param designMat a design Matrix
#' @param contrastsName the contrast to perform
#' @return a table of differential expression
getDiffExpTable <- function(expMat,
                            designMat,
                            contrastsName) {
    fits <- limma::lmFit(expMat, designMat$design)

    colnames(fits$coefficients) <- gsub("-", "_", colnames(fits$coefficients))

    contrast <- limma::makeContrasts(contrasts = c(contrastsName),
                                    levels = colnames(fits$coefficients))

    fitContrast <- limma::contrasts.fit(fits, contrasts = contrast)
    fitContrast <- limma::eBayes(fitContrast)
    tT <- limma::topTable(fitContrast, adjust = "fdr", sort.by = "p",
                    number = Inf)
    tT$contrast <- contrastsName

    return(tT)
}


.constrastHelper <- function(covariates, condition, contrastConds) {
    conditionsTypes <- as.character(unique(covariates[, condition]))
    if (is.na(contrastConds)) {
        if (length(conditionsTypes) > 2) {
            stop("Please compare only 2 conditions at once.")
        }
        cond1 <- paste0(condition, conditionsTypes[1])
        cond2 <- paste0(condition, conditionsTypes[2])
        contrastsName <- paste0(cond1, "-", cond2)
    } else {
        contrastsName <- contrastConds
    }
    return(contrastsName)
}

.pathwayCleaner <- function(pathways, id, geneCounts, minPathSize) {
    # select pathways with genes in the count data and a minimum pathway size
    pathways <- as.data.frame(pathways)
    genesPathways <- pathways[pathways[, id] %in% rownames(geneCounts), ]

    genesPathways <- genesPathways |> dplyr::group_by(Pathway) |>
        dplyr::summarise(n = dplyr::n()) |>
        dplyr::filter(n >= minPathSize)

    pathways <- pathways |>
        dplyr::filter(Pathway %in% genesPathways$Pathway)
    return(pathways)
}

.qNormalizer <- function(quantileNorm, pathSumStats, covariates) {

    if (quantileNorm == TRUE) {
        pathwayNames <- rownames(pathSumStats)
        pathSumStats <-
            preprocessCore::normalize.quantiles(pathSumStats)

        rownames(pathSumStats) <- pathwayNames
        colnames(pathSumStats) <- rownames(covariates)
    }
    return(pathSumStats)
}


.dirChecks <- function(outDir) {
    if (substring(outDir, nchar(outDir)) != "/") {
        outDir <- paste0(outDir, "/")
    }
    if (!dir.exists(outDir)) {
        stop("Output directory does not exist.")
    }
    return(outDir)
}

.pathSumCleaner <- function(pathSumStats) {
    # filter pathways with na values in the pathway summary statistics
    pathSumStats <- pathSumStats[rowSums(is.na(pathSumStats)) == 0, ]
    pathSumStats <- apply(pathSumStats, 2, function(x) {
        (x - mean(x)) / stats::sd(x)
    })
    return(pathSumStats)
}

.tScoreMaker <- function(deGenes, id) {
    tScores <- NULL
    if (!is.null(deGenes)) {
        tScores <- deGenes |>
            dplyr::mutate(!!id := rownames(deGenes)) |>
            dplyr::select(c(ENSEMBL, t))
    }
    return(tScores)
}
####
#
# Helper functions for miRNAPathwayEnrichment
#
####

#' Publication-ready miRNA-Pathway Enrichment table
#'
#' This function summarizes the outputs
#'
#' @param enrichmentTable Outputs from [miRNAPathwayEnrichment()] function
#' @return A summarized miRNA-Pathway enrichment table
#' @examples
#' data(msigdb_c2)
#' data(targetScan_03)
#' eTab <- miRNAPathwayEnrichment(targetScan_03[1:20],msigdb_c2[1:20])
#'
#' repTab <- reportEnrichment(eTab)
#' @export
reportEnrichment <- function(enrichmentTable) {
    cleanTab <- enrichmentTable[, c("x", "y", "pval")]
    cleanTab$pAdjust <- stats::p.adjust(cleanTab$pval)
    cleanTab <- dplyr::mutate_if(cleanTab, is.numeric, signif, digits = 3)
    cleanTab <- cleanTab[order(cleanTab$pval), ]
    names(cleanTab)[seq_len(2)] <- c("miRNA", "Pathway")
    return(cleanTab)
}

#' function to align a list of sets and a reference universe
#'
#' @param pathwaySets a list of sets
#' @param universe all set  elements must be a subset of universe
#' @return a list of sets, aligned to universe
alignToUniverse <- function(pathwaySets,
                            universe) {
    pathwaySets <- lapply(pathwaySets, function(x) {
        x[x %in% universe]
    })
    return(pathwaySets)
}


#' Pairwise enrichment analysis between two given lists of sets
#'
#' @param mirSets a list of targets of miRNAs
#' @param pathwaySets a list of pathways
#' @param pathsRef universe of genes.
#' @param numCores number of cores to calculate the results.
#' @return enrichment analysis results
enrichAllPairs <- function(mirSets,
                            pathwaySets,
                            pathsRef,
                            numCores) {
    iterator <- (merge(names(mirSets), names(pathwaySets)))
    iterator <- iterator |> dplyr::mutate_all(as.character)
    all <- length(pathsRef)

    # find enrichment p-value of each miRNA target set and each pathway set
    enrichs <- parallel::mclapply(seq_len(nrow(iterator)),
                    function(y) {
                        x <- iterator[y, ]
                        q <- length(intersect(unlist(pathwaySets[x[[2]]]),
                                            unlist(mirSets[x[[1]]])))
                        m <- length(unlist(mirSets[x[[1]]]))
                        n <- all - m
                        k <- length(unlist(pathwaySets[x[[2]]]))
                        pval <- stats::phyper(
                            q - 1, m, n, k, lower.tail = FALSE, log.p = FALSE)

                        return(c("pval" = pval, "Intersect" = q,
                            "mirset_Size" = m,
                            "not_mirset" = n,
                            "pathway_Size" = k,
                            "ratio_in" = q / (k - q),
                            "ratio_out" = (m - q) / n,
                            "ratio_ratios" = (q / (k - q)) / ((m - q) / n)))
                        }, mc.cores = numCores)

    temp <- do.call(rbind, enrichs)
    temp <- as.data.frame(temp)
    iterator <- cbind(iterator, temp)

    return(iterator)
}

####
#
# Helper functions for mappingPathwayClusters
#
####


#' Creates a network out of pcxn table
#'
#' @param pcxn pathways network edge list of pathways
#' @param edgeFDR FDR threshold for pathway-pathway adjusted p-values;
#'   filter edges with adjusted p-values less than given threshold
#' @param correlationCutOff cut-off threshold for pathway-pathway correlation;
#'   filter pathways with correlation less than given threshold
#' @param weighted True if you wish to include correlation weights in clustering
#' @return enrichment analysis results
pcxnToNet <- function(pcxn,
                        edgeFDR,
                        correlationCutOff,
                        weighted) {
    pcxn$pAdjust <- stats::p.adjust(pcxn$p.value, method = "fdr")
    res <- pcxn[pcxn$pAdjust < edgeFDR, ]
    res <- res[abs(res$PathCor) > correlationCutOff, ]
    allPathways <- union(unique(res$Pathway.B), unique(res$Pathway.A))
    net <- res[, c("Pathway.A", "Pathway.B", "PathCor")]
    if (weighted) {
        net$weight <- abs(net$PathCor)
    }
    net <- as.matrix(net)
    net <- igraph::graph_from_data_frame(net, directed = FALSE)

    igraph::set_vertex_attr(net, "col", value = "red")
    return(list("net" = net, "allPathways" = allPathways))
}

.levelFixerMapper <- function(clusts) {
    zz <- as.factor(clusts$membership)
    zz <- forcats::fct_infreq(zz, ordered = NA)
    levels(zz) <- seq_along(unique(zz))
    clusts$membership <- as.numeric(zz)
    return(clusts)
}

.shapeColNet <- function(subNet, dePathways1) {
    igraph::E(subNet)$color <-
        ifelse(as.numeric(igraph::E(subNet)$PathCor) > 0, "#E41A1C", "#377EB8")

    shapeIndex <- rownames(dePathways1) %in% igraph::V(subNet)$name
    shapeIndex <- dePathways1[shapeIndex, c("logFC", "AveExpr")]
    shapeIndex <- shapeIndex[igraph::V(subNet)$name, ]
    shapeIndex <- shapeIndex$logFC > 0
    igraph::V(subNet)$shape   <- ifelse(shapeIndex, "square", "circle")
    return(subNet)
}


####
#
# Helper functions for prioritizeMicroRNAs
#
####

.checkAddressDirs <- function(outDir, saveCSV, dataDir, saveSampling) {
    if (!dir.exists(outDir) && saveCSV) {
        warning("Output directory does not exist.")
        dir.create(outDir, recursive = TRUE)
    }
    if (!dir.exists(dataDir) && saveSampling) {
        stop("Data directory does not exist.")
    }
}

.cleanEnrichInput <- function(enriches0, enrichmentFDR) {
    enriches <- enriches0 |> dplyr::filter(Intersect != 0)
    enriches <- enriches |> dplyr::group_by(y) |>
        dplyr::mutate(path_fdr = stats::p.adjust(pval, method = "fdr"))
    enriches <- enriches |>
        dplyr::mutate(hit = ifelse(path_fdr < enrichmentFDR, 1, 0))
    return(enriches)
}


.makeSelector <- function(enriches, pathways) {
    # Count miRNA-pathway enrichment
    tempEnrich <- enriches[enriches$y %in% pathways, ]
    selector <- tempEnrich |>
        dplyr::group_by(x) |>
        dplyr::summarise("cluster_hits" = sum(hit))
    return(selector)
}


.saveCsvWriter <- function(saveCSV, prefix, sampRate, clustNo, selector,
                        outDir) {
    if (saveCSV == TRUE) {
        saveName <- paste0(prefix, sampRate, "_samples_clustNo_", clustNo,
                            ".csv")
        sayThis <- paste0(saveName, " saved!")
        message(sayThis)
        utils::write.csv(selector, paste0(outDir, saveName))
    }
}


.mkSampleDatDir <- function(dataDir, prefix, saveSampling, m, sampRate) {
    samplingDataDir <- paste0(dataDir, prefix, "Sampling_Data/")

    if ((saveSampling == TRUE) && (!dir.exists(samplingDataDir))) {
        dir.create(samplingDataDir, recursive = TRUE)
    }
    tempName <- paste0(prefix, m, "_", sampRate, "_samples.RDS")
    samplingDataFile <- paste0(samplingDataDir, "/", tempName)

    return(samplingDataFile)
}

.mSelectorMaker <- function(m, enrichNull, mSelector, sampRate, fn, nPaths,
                        sampDatFile, saveSampling, numCores, autoSeed, coverFn,
                        runJackKnife, pathways) {

    if (m %in% c("aggInv", "aggLog")) {
        samplingData <- samplingDataBase(enrichNull, mSelector,
                        sampRate, fn, nPaths, sampDatFile, jackKnife = FALSE,
                        saveSampling, numCores = numCores, autoSeed = autoSeed)
        mSelector <- methodProbBase(
            samplingData[[paste0("SampSize_", 100)]], mSelector,
            m = m, nPaths = nPaths, coverFn = coverFn)
    } else {
        names(mSelector)[2:3] <- paste0(m, "_", names(mSelector)[2:3])
    }
    sayThis <- paste0(m, " Method Done")
    message(sayThis)
    # perform jack-knife

    if ((runJackKnife == TRUE) && (m != "sumz") && (m != "sumlog")) {

        samplingData <- samplingDataBase(enrichNull, mSelector, sampRate, fn,
                    nPaths, sampDatFile, jackKnife = FALSE, saveSampling,
                    numCores = numCores, autoSeed = autoSeed)

        mSelector <- jackKnifeBase(selector = mSelector, pathways = pathways,
                    enrichNull = enrichNull, fn = fn,
                    jackKnifeData =  samplingData[[paste0("SampSize_", 100)]],
                    m = m, numCores = numCores)

        sayThis <- paste0(m, " JackKnifing Method Done!")
        message(sayThis)

        mSelector <- mSelector[, c(1, 3, 2, 4)]
    } else {
        mSelector <- mSelector[, c(1, 3, 2)]
    }
    return(mSelector)
}


.expandSelector <- function(selector, mSelector, m) {

    selector <- merge(selector, mSelector, all = TRUE)
    met <- paste0(m, "_pval")
    selector <- selector |> dplyr::arrange(!!rlang::sym(met))
    met2 <- paste0(m, "_fdr")
    selector <- selector |> dplyr::mutate(
        !!met2 := stats::p.adjust(p = !!rlang::sym(met), method = "fdr")
    )
    return(selector)
}
####
#
# Helper functions for clusterPlotter
#
####

.clusterPlotHelper <- function(plotSave, figDir, subNet, nodeColors,
                                legendCats) {
    if (plotSave) {
        grDevices::pdf(paste0(figDir, "PCxNCorGraph.pdf"),
                    width = 18, height = 11)
    }
    plot(subNet, vertex.size = 5, vertex.label = NA, vertex.color = nodeColors)
    graphics::legend(x = "bottomleft", legend = legendCats$attr, pch = c(0, 1),
                    bty = "n", cex = 1.6)
    graphics::legend(x = "topleft", legend = c("Positive Cor", "Negative Cor"),
                    col = c("#E41A1C", "#377EB8"), lty = 1, lwd = 2,
                    cex = 1.6, bty = "n")
    if (plotSave)
        grDevices::dev.off()
}

.clusterSubPlotHelper <- function(subplot, topClusters, clstMems, subNet,
                                figDir, cols, legendCats) {
    if (subplot == TRUE) {
        for (k in seq_len(topClusters)) {
            keep <- which((clstMems) == k)
            subNet2 <- igraph::induced_subgraph(subNet, keep)
            if (length(igraph::V(subNet2)) < 2) next

            grDevices::pdf(paste0(figDir, "PCxNCorGraph_",
                            "Cluster_", k, ".pdf"))
            plot(subNet2, edge.width = 1.3, vertex.size = 5, vertex.label = NA,
                vertex.color = cols[clstMems[keep]], legend = TRUE,
                layout = igraph::layout.fruchterman.reingold)
            graphics::legend(x = "bottomleft", legend = legendCats$attr,
                            pch = c(0, 1), bty = "n", cex = 1.4)
            graphics::legend(x = "topleft",
                            legend = c("Positive Cor", "Negative Cor"),
                            col = c("#E41A1C", "#377EB8"),
                            lty = 1, lwd = 2, cex = 1.4, bty = "n")
            grDevices::dev.off()
        }
    }
}

test_that("multiplication works", {
  expect_equal(2 * 2, 4)
})

test_that("Pathway-gene table works", {
    pathway1 <- c("125", "3099", "126")
    pathway2 <- c("5232", "5230", "5162")
    pathList <- list("Path1" = pathway1, "Path2" = pathway2)

    # org.Hs.eg.db is required
    res <- pathwayGeneTab(pathwayList = pathList)
    expect_equal(nrow(res), 6)
})


test_that("Pathway summary generates a correct number of pathways", {

    pathTab <- tibble::tribble(
        ~Pathway, ~ENTREZID,  ~ENSEMBL,
        "Path1",   "125",      "ENSG00000196616",
        "Path1",   "3099",     "ENSG00000159399",
        "Path2",   "5230",     "ENSG00000102144",
        "Path2",   "5162",     "ENSG00000168291"
    )

    exprsMat <- matrix(2 * (seq_len(12)), 4, 3)
    rownames(exprsMat) <- pathTab$ENSEMBL
    colnames(exprsMat) <- LETTERS[seq_len(3)]

    res <- pathwaySummary(exprsMat, pathTab, method = "x2")

    expect_equal(nrow(res), 2)
})


test_that("Pathway summary generates a correct number of samples", {

    pathTab <- tibble::tribble(
        ~Pathway, ~ENTREZID,  ~ENSEMBL,
        "Path1",   "125",      "ENSG00000196616",
        "Path1",   "3099",     "ENSG00000159399",
        "Path2",   "5230",     "ENSG00000102144",
        "Path2",   "5162",     "ENSG00000168291"
    )

    exprsMat <- matrix(2 * seq_len(1:12), 4, 3)
    rownames(exprsMat) <- pathTab$ENSEMBL
    colnames(exprsMat) <- LETTERS[seq_len(3)]

    res <- pathwaySummary(exprsMat, pathTab, method = "x2")

    expect_equal(ncol(res), 3)
})


test_that("Pathway summary generates correct activity scores", {

    pathTab <- tibble::tribble(
        ~Pathway, ~ENTREZID,  ~ENSEMBL,
        "Path1",   "125",      "ENSG00000196616",
        "Path1",   "3099",     "ENSG00000159399",
        "Path2",   "5230",     "ENSG00000102144",
        "Path2",   "5162",     "ENSG00000168291"
    )

    exprsMat <- matrix(2 * (1:12), 4, 3)
    rownames(exprsMat) <- pathTab$ENSEMBL
    colnames(exprsMat) <- LETTERS[1:3]

    res <- pathwaySummary(exprsMat, pathTab, method = "x2")

    expect_equal(res[1, 1], 2.5)
    expect_equal(res[2, 1], 12.5)
})

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----installation_bioc, eval = FALSE------------------------------------------
# if (!requireNamespace("BiocManager", quietly = TRUE))
#     install.packages("BiocManager")
# 
# BiocManager::install("PanomiR")

## ----installation_git, eval = FALSE-------------------------------------------
# devtools::install_github("pouryany/PanomiR")

## ----load_package-------------------------------------------------------------
library(PanomiR)

# Pathway reference from the PanomiR package
data("path_gene_table")
data("miniTestsPanomiR")
# Generating pathway summary statistics 

summaries <- pathwaySummary(miniTestsPanomiR$mini_LIHC_Exp,
                            path_gene_table, method = "x2",
                            zNormalize = TRUE, id = "ENSEMBL")

head(summaries)[,1:2]

## ----differential-------------------------------------------------------------

output0 <- differentialPathwayAnalysis(
                        geneCounts = miniTestsPanomiR$mini_LIHC_Exp,
                        pathways =  path_gene_table,
                        covariates = miniTestsPanomiR$mini_LIHC_Cov,
                        condition = 'shortLetterCode')

de.paths <- output0$DEP

head(de.paths,3)

## ----pcxn---------------------------------------------------------------------


# using an updated version of pcxn 

set.seed(2)
pathwayClustsLIHC <- mappingPathwaysClusters(
                            pcxn = miniTestsPanomiR$miniPCXN, 
                            dePathways = de.paths[1:300,],
                            topPathways = 200,
                            outDir=".",
                            plot = FALSE,
                            subplot = FALSE,
                            prefix='',
                            clusteringFunction = "cluster_louvain",
                            correlationCutOff = 0.1)


head(pathwayClustsLIHC$Clustering)


## ----miRNA--------------------------------------------------------------------

set.seed(1)
output2 <- prioritizeMicroRNA(enriches0 = miniTestsPanomiR$miniEnrich,
                    pathClust = miniTestsPanomiR$miniPathClusts$Clustering,
                    topClust = 1,
                    sampRate = 50, 
                    method = c("aggInv"),
                    outDir = "Output/",
                    dataDir = "outData/",
                    saveSampling = FALSE,
                    runJackKnife = FALSE,
                    numCores = 1,
                    prefix = "outmiR",
                    saveCSV = FALSE)

head(output2$Cluster1)


## ----customized_mir, eval = FALSE---------------------------------------------
# 
# 
# # using an updated version of pcxn
# data("msigdb_c2")
# data("targetScan_03")
# 
# 
# customeTableEnrich <- miRNAPathwayEnrichment(mirSets = targetScan_03,
#                                               pathwaySets = msigdb_c2,
#                                               geneSelection = yourGenes,
#                                               mirSelection = yourMicroRNAs,
#                                               fromID = "ENSEMBL",
#                                               toID = "ENTREZID",
#                                               minPathSize = 9,
#                                               numCores = 1,
#                                               outDir = ".",
#                                               saveOutName = NULL)
# 

## ----customized_mir2----------------------------------------------------------

# using an updated version of pcxn 
data("msigdb_c2")
data("targetScan_03")

tempEnrich <-miRNAPathwayEnrichment(targetScan_03[1:30],msigdb_c2[1:30])

head(reportEnrichment(tempEnrich))

## ----customized_gsc-----------------------------------------------------------
data("gscExample")

newPathGeneTable <-tableFromGSC(gscExample)


## ----customized_gsc2, eval = FALSE--------------------------------------------
# 
# library(GSEABase)
# 
# yourGeneSetCollection <- getGmt("YOUR GMT FILE")
# newPathGeneTable      <- tableFromGSC(yourGeneSetCollection)
# 

## ----sessionInfo--------------------------------------------------------------
sessionInfo()

#' Plots clusters of pathways with associated directionality.
#'
#' @param subNet pathways network (edge list of pathways)
#' @param subplot if TRUE, store individual clusters plots and connected plots
#'   in Figures directory of plots
#' @param topClusters plot figures for top x clusters
#' @param outDir output directory
#' @param prefix add prefix to plots
#' @param plotSave saves the plot if set true. Otherwise display
#' @return a set of plots for DE-PCXN and subclusters
#' @examples
#' data(miniTestsPanomiR)
#' clusterPlot(miniTestsPanomiR$miniPathClusts$DE_PCXN, plotSave = FALSE)
#' @export
clusterPlot <- function(subNet, subplot = FALSE, topClusters = 2, prefix = "",
                        outDir = ".", plotSave = TRUE) {
    if (!dir.exists(outDir)) {
        stop("Output directory does not exist.")
        }
    figDir <- paste0(outDir, "/", prefix)
    legendCats <- data.frame(attr = c("Up-regulated", "Down-regulated"),
                shape = unique(igraph::V(subNet)$shape))

    cols <- RColorBrewer::brewer.pal(8, "Set2")
    clstMems  <- igraph::V(subNet)$cluster
    nodeColors <- cols[clstMems]
    smallClust <- which(table(clstMems) <= table(clstMems)[5], useNames = TRUE)

    nodeColors[clstMems %in% smallClust] <- NA

    .clusterPlotHelper(plotSave, figDir, subNet, nodeColors, legendCats)

    .clusterSubPlotHelper(subplot, topClusters, clstMems, subNet,
                        figDir, cols, legendCats)

    "%notin%" <- Negate("%in%")
    remove <- which(table(clstMems) < 4)
    remove <- which((clstMems %notin% remove))
    subNet2 <- igraph::induced_subgraph(subNet, remove)

    if (subplot == TRUE) {
        grDevices::pdf(paste0(figDir, "ConnectedPathways_PCxNCorGraph.pdf"),
            width = 18, height = 11)
        plot(subNet2, edge.width = 1.3, vertex.size = 5, vertex.label = NA,
            vertex.color = cols[clstMems[remove]], legend = TRUE,
            layout = igraph::layout_components)
        graphics::legend(x = "bottomright", y = 200, legend = legendCats$attr,
            pch = c(0, 1), bty = "n", cex = 1.4)
        graphics::legend(x = "topleft",
            legend = c("Positive Cor", "Negative Cor"),
            col = c("#E41A1C", "#377EB8"), lty = 1, lwd = 2, cex = 1.4,
            bty = "n")
        grDevices::dev.off()
    }
}

#' Prioritize miRNA
#'
#' Outputs a table of miRNA ordered with respective p-values derived from
#'   method for prioritization
#'
#' @param enriches0 miRNA-pathway enrichment dataset obtained from
#'   miRNAPathwayEnrichment.
#' @param pathClust Pathway clusters, obtained from
#'   MappingPathwaysClusters.
#' @param method Vector of methods pCut, AggInv, AggLog, sumz, sumlog.
#' @param methodThresh Vector of methods threshold for each method in method,
#'   if NULL use default thresh values in method.
#' @param enrichmentFDR FDR cut-off calculating miRNA-pathway hits
#'   in the input cluster based on significant enrichment readouts.
#' @param topClust Top x clusters to perform miRNA prioritization on.
#' @param sampRate Sampling rate for CLT.
#' @param numCores Number of CPU cores to use, must be at least one.
#' @param outDir Output directory.
#' @param dataDir Data directory.
#' @param saveCSV If TRUE, saves CSV file for each cluster in topClust in
#'   outDir.
#' @param saveSampling If TRUE, saves sampling data as RDS for each cluster in
#'   topClust in dataDir.
#' @param runJackKnife If TRUE, jacknifing will be performed.
#' @param saveJackKnife If TRUE, saves jack-knifed sampling data as RDS for each
#'   cluster in topClust in dataDir.
#' @param prefix Prefix for all saved data.
#' @param autoSeed random permutations are generated based on predetermined
#'   seeds. TRUE will give identical results in different runs.
#' @return Table of miRNA and p-values, each row contains a miRNA and its
#'   associated p-values from the methods.
#' @examples
#' data("miniTestsPanomiR")
#'
#' prioritizeMicroRNA(enriches0 = miniTestsPanomiR$miniEnrich,
#'    pathClust = miniTestsPanomiR$miniPathClusts$Clustering,
#'    topClust = 1,
#'    sampRate = 50,
#'    method = c("aggInv"),
#'    saveSampling = FALSE,
#'    runJackKnife = FALSE,
#'    numCores = 1,
#'    saveCSV = FALSE)
#' @export
prioritizeMicroRNA <- function(enriches0, pathClust, method = "AggInv",
            methodThresh = NULL, enrichmentFDR = 0.25, topClust = 2,
            sampRate = 1000, outDir = ".", dataDir = ".", saveSampling = TRUE,
            runJackKnife = TRUE, saveJackKnife = FALSE, numCores = 1,
            saveCSV = TRUE, prefix = "", autoSeed = TRUE) {
    .checkAddressDirs(outDir, saveCSV, dataDir, saveSampling)
    output   <- list()
    enriches <- .cleanEnrichInput(enriches0, enrichmentFDR)
    for (clustNo in seq_len(topClust)) {
        clustName <- paste0("Cluster", clustNo)
        sayThis <- paste0("Working on ", clustName, ".")
        message(sayThis)
        pathways <- as.character(pathClust[pathClust$cluster == clustNo,
                                    ]$Pathway)
        nPaths <- length(pathways)
        selector <- .makeSelector(enriches, pathways)
        for (i in seq_along(method)) {
            m <- method[i]
            sayThis <- paste0("Performing ", m, " function.")
            message(sayThis)
            fn <- get(paste0(m, "Fn"))
            coverFn <- get(paste0(m, "CoverFn"))
            if (!is.null(methodThresh)) {
                mThresh <- methodThresh[i]
                temp <- fn(enriches = enriches0, pathways, isSelector = TRUE,
                    thresh = mThresh)
            } else {
                temp <- fn(enriches = enriches0, pathways, isSelector = TRUE)
            }
            mSelector   <- temp$selector
            mEnriches0  <- temp$enriches0
            if (nrow(mSelector) < 3) {
                sayThis <- paste0("Skipping ", m, " function:few miRNAs")
                message(sayThis)
                next
            }
            enrichNull <- mEnriches0 |> dplyr::filter(x %in% mSelector$x)
            sampDatFile <- .mkSampleDatDir(dataDir, prefix, saveSampling,
                                        m, sampRate)
            mSelector   <- .mSelectorMaker(m, enrichNull, mSelector, sampRate,
                        fn, nPaths, sampDatFile, saveSampling, numCores,
                        autoSeed, coverFn, runJackKnife, pathways)
            selector <- .expandSelector(selector, mSelector, m)
        }
        .saveCsvWriter(saveCSV, prefix, sampRate, clustNo, selector, outDir)
        output[[clustName]] <- selector
    }
    return(output)
}

#' Differential Expression Analysis For Pathways
#'
#' Performs differential expression analysis for pathways using LIMMA package
#' with gene counts
#'
#' @param geneCounts Gene counts, rows refer to genes and columns to samples.
#' @param pathways Pathways table, containing pathway names and genes with id
#'   specified.
#' @param covariates Covariates/metadata file; rows matches the columns of
#'   geneCounts.
#' @param condition Condition to be examined (tumor vs normal etc); must exist
#'   in covariates column.
#' @param adjustCovars Adjustment covariates like batch; if NULL,
#'   no adjustments performed.
#' @param covariateCorrection If TRUE, performs covariates detection and
#'   correction; requires **adjustCovars**; (limma).
#' @param quantileNorm If TRUE, performs quantile normalization on pathway
#'   summary statistics; from *preprocess* package.
#' @param outDir Output directory.
#' @param saveOutName If not NULL, saves output as RDS using save name,
#'   if NULL, does not save output.
#' @param id ID matching genes to pathways; rownames of geneCounts.
#' @param deGenes If not NULL, add t-scores to pathways summary statistics;
#'   filter by genes t-scores.
#' @param minPathSize Minimum pathway size.
#' @param method Define method to use for pathway summary statistics;
#'   specifications in documentations.
#' @param trim Filter pathways with mean less than trim threshold
#'   in pathway summary statistics.
#' @param geneCountsLog If TRUE, log(geneCounts).
#' @param contrastConds Provide a contrast expression to be used in Limma
#'   comparison. This is necessary if you have more than two levels in the
#'   condition covariate.
#' @return List containing differentially expressed pathways as DEP and pathway
#'   summary statistics as pathwaySummaryStats.
#' @examples
#'
#' data("path_gene_table")
#' data("miniTestsPanomiR")
#'
#' differentialPathwayAnalysis(geneCounts = miniTestsPanomiR$mini_LIHC_Exp,
#' pathways =  path_gene_table,
#' covariates = miniTestsPanomiR$mini_LIHC_Cov,
#' condition = 'shortLetterCode')
#'
#' @export
differentialPathwayAnalysis <- function(geneCounts, pathways, covariates,
            condition, adjustCovars = NULL, covariateCorrection = FALSE,
            quantileNorm = FALSE, outDir = ".", saveOutName = NULL,
            id = "ENSEMBL", deGenes = NULL, minPathSize = 10, method = "x2",
            trim = 0.025, geneCountsLog = TRUE, contrastConds = NA) {

    outDir   <- .dirChecks(outDir)
    pathways <- .pathwayCleaner(pathways, id, geneCounts, minPathSize)
    tScores  <- .tScoreMaker(deGenes, id)

    if (geneCountsLog == TRUE) {
        geneCounts <- log(geneCounts)
    }

    pathSumStats <- pathwaySummary(geneCounts, pathways, id, zNormalize = FALSE,
        method = method,  deGenes = deGenes, trim = trim, tScores = tScores)
    pathSumStats <- .pathSumCleaner(pathSumStats)
    pathSumStats <- .qNormalizer(quantileNorm, pathSumStats, covariates)

    fitResiduals <- NULL
    if (covariateCorrection == TRUE) {
        stop("Under development. Please use covariateCorrection = FALSE option")
    } else {
        if (is.null(adjustCovars)) {
            conditionsDat <- as.data.frame(covariates[, condition])
            colnames(conditionsDat) <- condition
            rownames(conditionsDat) <- rownames(covariates)
            designMat <- getDesignMatrix(conditionsDat, intercept = FALSE)
        } else {
            designMat <- getDesignMatrix(covariates[, c(condition,
                            adjustCovars), drop = FALSE], intercept = FALSE)
            designMat$design <-
                designMat$design[, linColumnFinder(designMat$design)$indepCols]
            fitResiduals <- getResidual(covariates, adjustCovars, pathSumStats)
        }
    }
    contrastsName <- .constrastHelper(covariates, condition, contrastConds)
    tT            <- getDiffExpTable(pathSumStats, designMat, contrastsName)
    output        <- list("DEP" = tT, "pathwaySummaryStats" = pathSumStats,
                "contrast" = contrastsName, "PathwayResiduals" = fitResiduals)

    if (!is.null(saveOutName)) {
        saveRDS(output, paste0(outDir, saveOutName))
    }
    return(output)
}

#' Canonical pathways from Molecular Signatures Database, MsigDb V6.2
#'
#' @format A list of 1143 pathways
#' @source \url{http://www.gsea-msigdb.org/gsea/index.jsp}
#' @usage data(msigdb_c2)
#' @examples
#' data(msigdb_c2)
"msigdb_c2"

#' Example genesets from MSigDB
#'
#' @format A GeneSet Collection object containing two genesets.
#' @source \url{http://www.gsea-msigdb.org/gsea/index.jsp}
#' @usage data(gscExample)
#' @examples
#' data(gscExample)
"gscExample"

#' A table of gene-pathway association. based on the pathways of MSigDB.
#'
#' @format A matrix with 3 columns and 76926 rows:
#' \describe{
#'   \item{Pathway}{An MSigDB annotated pathway}
#'   \item{ENTREZID}{The ENTREZID of a gene belonging to the pathway}
#'   \item{ENSEMBL}{The ENSEMBL of a gene belonging to the pathway}
#' }
#' @usage data(path_gene_table)
#' @examples
#' data(path_gene_table)
"path_gene_table"

#' A processed list of miRNA target gene sets from the TargetScan dataset.
#' Each list item is a list of genes targeted by the respective miRNA family
#'
#' The interactions are filtered to only human interactions.
#'
#' The interactions are filtered to have a
#' Cumulative weighted context++ score of < -0.3
#' @format A list of 439 items
#' @source \url{http://www.targetscan.org/vert_72/}
#' @usage data(targetScan_03)
#' @examples
#' data(targetScan_03)
"targetScan_03"

#' Readouts and datasets for minimal reproducible examples of the PanomiR.
#'
#' The item miniEnrich is a reduced representation of the TargetScan
#' For full table use miRNAPathwayEnrichment function in the package
#' along with msigdb_c2 and targetScan_03 datasets.
#'
#' These datasets include reduced representation of TCGA LIHC data
#' for reproducing the pipeline. doi: 10.1016/j.cell.2017.05.046
#'
#' A reduced representation of PCxN is provided. For full dataset and method
#' please refer to pcxn.org or https://doi.org/10.1371/journal.pcbi.1006042
#'
#' @format A list of 5:
#' \describe{
#'   \item{mini_LIHC_Exp}{a reduced expression dataset from TCGA LIHC data}
#'   \item{mini_LIHC_Cov}{a reduced covariates dataset from TCGA LIHC data}
#'   \item{miniEnrich}{a reduced table of miRNA-pathway enrichment, TargetScan.}
#'   \item{miniDEP}{Differentially activated pathways from reduced TCGA LIHC}
#'   \item{miniPCXN}{reduced representation of PCXN network}
#'   \item{miniPathClusts}{miniDEP mapped to miniPCXN}
#' }
#' @usage data(miniTestsPanomiR)
#' @examples
#' data(miniTestsPanomiR)
"miniTestsPanomiR"

#' Enrichment Probability Of miRNAs
#'
#' Outputs enrichment probability of miRNAs based on pathway clusters.
#'
#' @param mirSets Table of miRNAs and a list of their interactions with
#'   genes in ENTREZ ID.
#' @param pathwaySets Table of pathways and a list of their interactions
#'   with genes in ENTREZ ID.
#' @param geneSelection Table of genes with dtype; if not NULL, select only
#'   genes from a given table.
#' @param mirSelection Table of miRNA names; if not NULL, select only miRNAs
#'   from given table.
#' @param fromID ID of genes in geneSelection.
#' @param toID ID of genes used in pcxn and pathways set.
#' @param minPathSize Filter out pathways with sets less than given value.
#' @param numCores Number of CPU cores to use, must be at least one.
#' @param outDir Output directory.
#' @param saveOutName If not NULL, saves output as RDS using save name.
#' @return Table of enrichment, each row contains mirna-pathway and its
#'   enrichment p-values.
#' @examples
#' data(msigdb_c2)
#' data(targetScan_03)
#' miRNAPathwayEnrichment(targetScan_03[1:20],msigdb_c2[1:20])
#' @export
miRNAPathwayEnrichment <- function(mirSets,
                                    pathwaySets,
                                    geneSelection = NULL,
                                    mirSelection = NULL,
                                    fromID = "ENSEMBL",
                                    toID = "ENTREZID",
                                    minPathSize = 9,
                                    numCores = 1,
                                    outDir = ".",
                                    saveOutName = NULL) {
    if (!dir.exists(outDir)) {
        stop("Output directory does not exist.")
    }

    # select pathways with minimum set size
    pathsSel <- vapply(pathwaySets, length, numeric(1))
    pathwaySets <- pathwaySets[pathsSel > minPathSize]
    pathsRef <- Reduce(union, pathwaySets)

    # select miRNAs with targets in pathways
    mirSets <- alignToUniverse(mirSets, pathsRef)
    # select pathways with selected genes of interest
    if (!is.null(geneSelection)) {
        geneDF <- clusterProfiler::bitr(geneSelection, fromType = fromID,
            toType = toID, OrgDb = org.Hs.eg.db::org.Hs.eg.db)

        pathwaySets <- alignToUniverse(pathwaySets, geneDF[, c(toID)])
        mirSets     <- alignToUniverse(mirSets, geneDF[, c(toID)])
        pathsRef    <- Reduce(union, pathwaySets)
    }
    # select miRNAs of interest
    if (!is.null(mirSelection)) {
        mirSets <- mirSets[names(mirSets) %in% mirSelection]
    }
    selVec   <- vapply(mirSets, length, numeric(1))
    mirSets  <- mirSets[selVec > minPathSize]

    iterator <- enrichAllPairs(mirSets, pathwaySets, pathsRef, numCores)

    if (!is.null(saveOutName)) {
        saveRDS(iterator, paste0(outDir, "/", saveOutName))
    }
    return(iterator)
}

---
title: "miRNA and pathway analysis with PanomiR"
author: "Pourya Naderi, Alan Teo, Ilya Sytchev, and Winston Hide"
output: 
  BiocStyle::html_document
bibliography: references.bib  
vignette: >
  %\VignetteIndexEntry{PanomiR Tutorial}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

# Introduction

MicroRNAs (miRNAs) can target co-expressed genes to coordinate multiple 
pathways. “Pathway networks of miRNA Regulation” (PanomiR) is a
framework to support the discovery of miRNA regulators based on their targeting
of coordinated pathways. It analyzes and prioritizes multi-pathway dynamics
of miRNA-orchestrated regulation, as opposed to investigating isolated
miRNA-pathway interaction events. PanomiR uses predefined pathways, their 
co-activation, gene expression, and annotated miRNA-mRNA interactions to extract
miRNA-pathway targeting events. This vignette describes PanomiR’s functions
and analysis tools to derive these multi-pathway targeting events. 

If you use PanomiR for your research, please cite PanomiR's manuscript
[@yeganeh2022panomir]. Please send any questions/suggestions you may have to
`pnaderiy [at] bidmc [dot] harvard [dot] edu` or submit Github issues at
[https://github.com/pouryany/PanomiR]().

Naderi Yeganeh, Pourya, Yue Yang Teo, Dimitra Karagkouni,
Yered Pita-Juarez, Sarah L. Morgan, Ioannis S. Vlachos, and Winston Hide.
"PanomiR: A systems biology framework for analysis of multi-pathway targeting
by miRNAs." bioRxiv (2022). doi: [https://doi.org/10.1101/2022.07.12.499819]().

# Installation

PanomiR can be accessed via Bioconductor. To install, start R (version >= 4.2.0)
and run the following code.

```{r installation_bioc, eval = FALSE}
if (!requireNamespace("BiocManager", quietly = TRUE))
    install.packages("BiocManager")

BiocManager::install("PanomiR")
```

You can also install the latest development version of PanomiR using GitHub.

```{r installation_git, eval = FALSE}
devtools::install_github("pouryany/PanomiR")
```

# Overview

PanomiR is a framework to prioritize disease-associated miRNAs using activity 
of disease-associated pathways. The input datasets for PanomiR are (a) a gene 
expression dataset along with covariates such as disease-state and batch,
(b) a background collection of pathways/genesets, and (c) a collection of
miRNAs and their gene targets.

The workflow of PanomiR includes (a) generation of pathway summary
statistics from gene expression data, (b) detection of differentially activated
pathways, (c) finding coherent groups, or clusters, of differentially activated
pathways, and (d) detecting miRNAs that target each group of pathways. 

Individual steps of the workflow can be used in isolation to carry out specific
analyses. The following sections outline each step and the material needed to
execute PanomiR. 

# Pathway summarization

PanomiR generates pathway activity summary profiles from gene expression data
and a list of pathways. Pathway summaries are numbers that represent the overall
activity of genes that belong to each pathway. These numbers are calculated
based on a methodology previously described in part by Altschuler et al.
[@altschuler2013pathprinting;@joachim2018relative].
Briefly, genes in each sample are ranked by their expression values and then
pathway summaries are calculated as the average rank-squared of genes within a 
pathway. The summaries are then center and scaled (zNormalized) across samples.

The default list of background pathways in PanomiR is formatted into a table
(`data("path_gene_table")`). The table is based on canonical pathways collection
of Molecular Signatures Database (MSigDB) V6.2 and it contains annotated 
pathways from a variety of sources  [@liberzon2011molecular].

** Users interested in using other pathway/geneset backgrounds, such as newer 
versions of MSigDB or KEGG, should refer to the [appendix](#geneset) of
this manual.

This section uses a reduced example dataset from The Cancer Genome Atlas (TCGA)
Liver Hepatocellular Carcinoma (LIHC) dataset to generate
pathway summary statistics [@ally2017comprehensive]. **Note:** Make sure that
you select a gene representation type that matches the rownames of your
expression data. The type can be modified using the `id` argument in the
function below. The default value for this argument is `ENSEMBL`. 

```{r load_package}
library(PanomiR)

# Pathway reference from the PanomiR package
data("path_gene_table")
data("miniTestsPanomiR")
# Generating pathway summary statistics 

summaries <- pathwaySummary(miniTestsPanomiR$mini_LIHC_Exp,
                            path_gene_table, method = "x2",
                            zNormalize = TRUE, id = "ENSEMBL")

head(summaries)[,1:2]
```




# Differential Pathway activation

Once you generate the pathway activity profiles, as discussed in the last
section, there are several possible analyses that you can perform. We have
bundled some of the most important ones into standalone functions. Here, we
describe differential pathway activity profiling to determine dysregulatd
pathways. This function analyzes differences in pathway activity profiles
in user-determined conditions.

At this stage you need to provide a pathway-gene association table, an
expression dataset, and a covariates table. You need to specify covariates that
you would like to contrast. You also need to provide a contrast, as formatted in
limma [@ritchie2015limma]. If the contrast is not provided, the function assumes
the first two levels of the provided covariate are to be contrasted.
**Note:** make sure the contrast covariate is formatted as factor.


```{r differential}

output0 <- differentialPathwayAnalysis(
                        geneCounts = miniTestsPanomiR$mini_LIHC_Exp,
                        pathways =  path_gene_table,
                        covariates = miniTestsPanomiR$mini_LIHC_Cov,
                        condition = 'shortLetterCode')

de.paths <- output0$DEP

head(de.paths,3)
```

# Finding clusters of pathways

PanomiR provides a function to find groups coordinated differentially 
activated pathways based on a pathway co-expression network (PCxN) previously
described in [@pita2018pathway]. Briefly, PCxN
is a network where nodes are pathways and links are co-expression between the
nodes. It is formatted into a table were rows represent edges. The edges of PCxN
are marked by two numbers, 1- a correlation co-efficient and 2- a significance
adjusted p-value. Cut-offs for both of these numbers can be manually set using 
PanomiR functions. See function manuals for more info. 

PCxN and its associated genesets are already released and can be accessed
through following Bioconductor packages:
[pcxn](http://bioconductor.org/packages/release/bioc/html/pcxn.html) and
[pcxnData](http://bioconductor.org/packages/release/data/experiment/html/pcxnData.html).

Here we have provided a small version of PCxN for tutorial purposes. A more 
recent version of PCxN based on MSigDB V6.2 is available through the 
data repository accompanying PanomiR manuscript, which can be found
[here](https://github.com/pouryany/PanomiR_paper).


```{r pcxn}


# using an updated version of pcxn 

set.seed(2)
pathwayClustsLIHC <- mappingPathwaysClusters(
                            pcxn = miniTestsPanomiR$miniPCXN, 
                            dePathways = de.paths[1:300,],
                            topPathways = 200,
                            outDir=".",
                            plot = FALSE,
                            subplot = FALSE,
                            prefix='',
                            clusteringFunction = "cluster_louvain",
                            correlationCutOff = 0.1)


head(pathwayClustsLIHC$Clustering)

```


# Prioritizing miRNAs per cluster of pathways.

PanomiR identifies miRNAs that target clusters of pathways, as defined in the 
last section. In order to this, you would need a reference table of
miRNA-Pathway association score (enrichment). We recommend using a customized
miRNA-Pathway association table, tailored to your experimental data.
This section provides an overview of prioritization process. Readers who 
interested in knowing more about the technical details of PanomiR can access
PanomiR's accompanying publication [@yeganeh2022panomir].

## Enrichment reference
Here, we provide a pre-processed small example table of miRNA-pathway enrichment
in `miniTestsPanomiR$miniEnrich` object. This table contains enrichment analysis
results using Fisher's Exact Test between MSigDB pathways and TargetScan miRNA
targets. The individual components are accessible via `data(msigdb_c2)` and
`data(targetScan_03)` [@agarwal2015predicting; @liberzon2011molecular]. This
example table contains only a subset of the full pairwise enrichment. 
You can refer to [section 5](#geneset) of this manual to learn how to create
enrichment tables and how to customize them to your specific gene expression
data.

## Generating targeting scores
PanomiR generates individual scores for individual miRNAs, which quantify
targeting a group of pathways. These scores are generated based on the reference
enrichment table described in the previous section. We are interested in knowing
to what extent each miRNA targets clusters of pathways identified in the last
step (see previous section). 

PanomiR constructs a null distribution of the targeting score for each miRNA.
It then contrasts observed scores from a given group of pathways (clusters)
against the null distribution in order to generate a targeting p-value.
These p-values are used to rank miRNAs per cluster.

## Sampling parameter
The process described above requires repeated sampling to empirically obtain the
null distribution. The argument `sampRate` denotes the number of repeats in the
process. Note that in the example below, we use a sampling rate of 50, the
recommended rate is between 500-1000. Also, we set the `saveSampling` argument
to `FALSE`. This argument, when set `TRUE`, ensures that the null distribution
is obtained only once. This argument should be set to TRUE if you wish to save
your sampling and check for different outputs from the clustering algorithms or
pathway thresholds.


```{r miRNA}

set.seed(1)
output2 <- prioritizeMicroRNA(enriches0 = miniTestsPanomiR$miniEnrich,
                    pathClust = miniTestsPanomiR$miniPathClusts$Clustering,
                    topClust = 1,
                    sampRate = 50, 
                    method = c("aggInv"),
                    outDir = "Output/",
                    dataDir = "outData/",
                    saveSampling = FALSE,
                    runJackKnife = FALSE,
                    numCores = 1,
                    prefix = "outmiR",
                    saveCSV = FALSE)

head(output2$Cluster1)

```



# miRNA-Pathway enrichment tables 

We recommend using PanomiR with on tissue/experiment-customized datasets.
In order to do this, you need to create a customized enrichment table.
You can simply do so by using the pathway and miRNA list that we have provided
as a part of the package. Simply, plug in the name of the genes that are present
(expressed) in your experiment in the following code:



```{r customized_mir, eval = FALSE}


# using an updated version of pcxn 
data("msigdb_c2")
data("targetScan_03")


customeTableEnrich <- miRNAPathwayEnrichment(mirSets = targetScan_03,
                                              pathwaySets = msigdb_c2,
                                              geneSelection = yourGenes,
                                              mirSelection = yourMicroRNAs,
                                              fromID = "ENSEMBL",
                                              toID = "ENTREZID",
                                              minPathSize = 9,
                                              numCores = 1,
                                              outDir = ".",
                                              saveOutName = NULL)

```

In the above section, the field ```fromID``` denotes the gene representation 
format of your input list. Here is a quick example that runs fast. Note that 
the `miRNAPathwayEnrichment()` function creates a detailed report with
parameters that are used internally. To get a smaller table that is suitable 
for publication purposes, use `reportEnrichment()` function.
```{r customized_mir2}

# using an updated version of pcxn 
data("msigdb_c2")
data("targetScan_03")

tempEnrich <-miRNAPathwayEnrichment(targetScan_03[1:30],msigdb_c2[1:30])

head(reportEnrichment(tempEnrich))
```

# Customized genesets and recommendations {#geneset}

PanomiR can integrate genesets and pathways from external sources including
those annotated in MSigDB. In order to do so, you need to provide a 
`GeneSetCollection` object as defined in the `GSEABase` package. 

The example below illustrates using external sources to create your 
own customized pathway-gene association table. This customized table can
replace the `path_gene_table` input in sections 1, 2, and 5
of this manual.

```{r customized_gsc}
data("gscExample")

newPathGeneTable <-tableFromGSC(gscExample)

```

The the pathway correlation network in section 3 is build upon an MSigDB V6.2, 
canonical pathways (cp) collection dataset that includes KEGG Pathways.
KEGG prohibits distribution of its pathways by third parties. You can
access desired versions of MSigDB in gmt format via
[this link](https://www.gsea-msigdb.org/gsea/downloads_archive.jsp)
[@subramanian2005gene].

The library `msigdb` provides an programmatic interface to download different
geneset collections. Including how to add KEGG pathways or download mouse 
genesets. Use the this [MSigDB tutorial](https://bioconductor.org/packages/release/data/experiment/vignettes/msigdb/inst/doc/msigdb.html)
to create your desired gene sets.

You can also use the following code chunk to create pathway-gene association
tables from gmt files.

```{r customized_gsc2, eval = FALSE}

library(GSEABase)

yourGeneSetCollection <- getGmt("YOUR GMT FILE")
newPathGeneTable      <- tableFromGSC(yourGeneSetCollection)

```

# Session info
```{r sessionInfo}
sessionInfo()

```

# References

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{enrichAllPairs}
\alias{enrichAllPairs}
\title{Pairwise enrichment analysis between two given lists of sets}
\usage{
enrichAllPairs(mirSets, pathwaySets, pathsRef, numCores)
}
\arguments{
\item{mirSets}{a list of targets of miRNAs}

\item{pathwaySets}{a list of pathways}

\item{pathsRef}{universe of genes.}

\item{numCores}{number of cores to calculate the results.}
}
\value{
enrichment analysis results
}
\description{
Pairwise enrichment analysis between two given lists of sets
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{pCutFn}
\alias{pCutFn}
\title{Score miRNAs In a Cluster Of Pathways}
\usage{
pCutFn(enriches, pathways, isSelector, thresh = 0.05)
}
\arguments{
\item{enriches}{Table of miRNA pathway enrichments.}

\item{pathways}{Queried pathways, e.g. cluster pathways.}

\item{isSelector}{Internal argument.}

\item{thresh}{Threshold from p-value cut-off.}
}
\value{
P-value based scoring of miRNAs in a cluster of pathways.
}
\description{
The function to count the number of enriched pathways for each miRNA.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mappingPathwaysClusters.R
\name{mappingPathwaysClusters}
\alias{mappingPathwaysClusters}
\title{Outputs a table with pathways and their respective clusters}
\usage{
mappingPathwaysClusters(
  pcxn,
  dePathways,
  clusteringFunction = NULL,
  edgeFDR = 0.05,
  correlationCutOff = 0.316,
  pathwayFDR = 0.05,
  topPathways = 200,
  plotOut = TRUE,
  subplot = TRUE,
  topClusters = 2,
  prefix = "",
  outDir = ".",
  saveNameCSV = NULL,
  weighted = FALSE
)
}
\arguments{
\item{pcxn}{pathways network (edge list of pathways)}

\item{dePathways}{differential expressed pathways, obtained from
*DifferentialPathwayAnalysis*}

\item{clusteringFunction}{clustering algorithm}

\item{edgeFDR}{FDR threshold for pathway-pathway adjusted p-values;
filter edges with adjusted p-values less than given threshold}

\item{correlationCutOff}{cut-off threshold for pathway-pathway correlation;
filter pathways with correlation less than given threshold}

\item{pathwayFDR}{FDR threshold for DE pathways adjusted p-values;
filter pathways with adjusted p-values less than given threshold}

\item{topPathways}{use only top x paths; if NULL, use all paths}

\item{plotOut}{if TRUE, store graph plot in Figures directory of plots}

\item{subplot}{if TRUE, store individual clusters plots and connected plots
in Figures directory of plots}

\item{topClusters}{plot figures for top x clusters}

\item{prefix}{add prefix to plots}

\item{outDir}{output directory}

\item{saveNameCSV}{if not NULL, saves output as csv using save name}

\item{weighted}{True if you wish to include correlation weights in clustering}
}
\value{
a list where the first item is a table with each row containing
  a pathway and its respective cluster. The second item is an igraph object.
}
\description{
Outputs a table with pathways and their respective clusters
}
\examples{
data("miniTestsPanomiR")

mappingPathwaysClusters(pcxn = miniTestsPanomiR$miniPCXN,
                         dePathways = miniTestsPanomiR$miniDEP,
                         topPathways = 200,
                         outDir=".",
                         plot = FALSE,
                         subplot = FALSE,
                         prefix='',
                         clusteringFunction = "cluster_louvain",
                         correlationCutOff = 0.1)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/miRNAPathwayEnrichment.R
\name{miRNAPathwayEnrichment}
\alias{miRNAPathwayEnrichment}
\title{Enrichment Probability Of miRNAs}
\usage{
miRNAPathwayEnrichment(
  mirSets,
  pathwaySets,
  geneSelection = NULL,
  mirSelection = NULL,
  fromID = "ENSEMBL",
  toID = "ENTREZID",
  minPathSize = 9,
  numCores = 1,
  outDir = ".",
  saveOutName = NULL
)
}
\arguments{
\item{mirSets}{Table of miRNAs and a list of their interactions with
genes in ENTREZ ID.}

\item{pathwaySets}{Table of pathways and a list of their interactions
with genes in ENTREZ ID.}

\item{geneSelection}{Table of genes with dtype; if not NULL, select only
genes from a given table.}

\item{mirSelection}{Table of miRNA names; if not NULL, select only miRNAs
from given table.}

\item{fromID}{ID of genes in geneSelection.}

\item{toID}{ID of genes used in pcxn and pathways set.}

\item{minPathSize}{Filter out pathways with sets less than given value.}

\item{numCores}{Number of CPU cores to use, must be at least one.}

\item{outDir}{Output directory.}

\item{saveOutName}{If not NULL, saves output as RDS using save name.}
}
\value{
Table of enrichment, each row contains mirna-pathway and its
  enrichment p-values.
}
\description{
Outputs enrichment probability of miRNAs based on pathway clusters.
}
\examples{
data(msigdb_c2)
data(targetScan_03)
miRNAPathwayEnrichment(targetScan_03[1:20],msigdb_c2[1:20])
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{alignToUniverse}
\alias{alignToUniverse}
\title{function to align a list of sets and a reference universe}
\usage{
alignToUniverse(pathwaySets, universe)
}
\arguments{
\item{pathwaySets}{a list of sets}

\item{universe}{all set  elements must be a subset of universe}
}
\value{
a list of sets, aligned to universe
}
\description{
function to align a list of sets and a reference universe
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{getDesignMatrix}
\alias{getDesignMatrix}
\title{Obtain Design Matrix}
\usage{
getDesignMatrix(covariatesDataFrame, intercept = TRUE, reLevels = list())
}
\arguments{
\item{covariatesDataFrame}{Dataframe of covariates.}

\item{intercept}{intercept in the linear model.}

\item{reLevels}{TBA.}
}
\value{
List containing a design matrix.
}
\description{
Modified from covariates pipeline of Menachem Former. Imported from
\url{https://github.com/th1vairam/CovariateAnalysis}
}
\examples{
data(iris)
getDesignMatrix(iris)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{msigdb_c2}
\alias{msigdb_c2}
\title{Canonical pathways from Molecular Signatures Database, MsigDb V6.2}
\format{
A list of 1143 pathways
}
\source{
\url{http://www.gsea-msigdb.org/gsea/index.jsp}
}
\usage{
data(msigdb_c2)
}
\description{
Canonical pathways from Molecular Signatures Database, MsigDb V6.2
}
\examples{
data(msigdb_c2)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/differentialPathwayAnalysis.R
\name{differentialPathwayAnalysis}
\alias{differentialPathwayAnalysis}
\title{Differential Expression Analysis For Pathways}
\usage{
differentialPathwayAnalysis(
  geneCounts,
  pathways,
  covariates,
  condition,
  adjustCovars = NULL,
  covariateCorrection = FALSE,
  quantileNorm = FALSE,
  outDir = ".",
  saveOutName = NULL,
  id = "ENSEMBL",
  deGenes = NULL,
  minPathSize = 10,
  method = "x2",
  trim = 0.025,
  geneCountsLog = TRUE,
  contrastConds = NA
)
}
\arguments{
\item{geneCounts}{Gene counts, rows refer to genes and columns to samples.}

\item{pathways}{Pathways table, containing pathway names and genes with id
specified.}

\item{covariates}{Covariates/metadata file; rows matches the columns of
geneCounts.}

\item{condition}{Condition to be examined (tumor vs normal etc); must exist
in covariates column.}

\item{adjustCovars}{Adjustment covariates like batch; if NULL,
no adjustments performed.}

\item{covariateCorrection}{If TRUE, performs covariates detection and
correction; requires **adjustCovars**; (limma).}

\item{quantileNorm}{If TRUE, performs quantile normalization on pathway
summary statistics; from *preprocess* package.}

\item{outDir}{Output directory.}

\item{saveOutName}{If not NULL, saves output as RDS using save name,
if NULL, does not save output.}

\item{id}{ID matching genes to pathways; rownames of geneCounts.}

\item{deGenes}{If not NULL, add t-scores to pathways summary statistics;
filter by genes t-scores.}

\item{minPathSize}{Minimum pathway size.}

\item{method}{Define method to use for pathway summary statistics;
specifications in documentations.}

\item{trim}{Filter pathways with mean less than trim threshold
in pathway summary statistics.}

\item{geneCountsLog}{If TRUE, log(geneCounts).}

\item{contrastConds}{Provide a contrast expression to be used in Limma
comparison. This is necessary if you have more than two levels in the
condition covariate.}
}
\value{
List containing differentially expressed pathways as DEP and pathway
  summary statistics as pathwaySummaryStats.
}
\description{
Performs differential expression analysis for pathways using LIMMA package
with gene counts
}
\examples{

data("path_gene_table")
data("miniTestsPanomiR")

differentialPathwayAnalysis(geneCounts = miniTestsPanomiR$mini_LIHC_Exp,
pathways =  path_gene_table,
covariates = miniTestsPanomiR$mini_LIHC_Cov,
condition = 'shortLetterCode')

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{path_gene_table}
\alias{path_gene_table}
\title{A table of gene-pathway association. based on the pathways of MSigDB.}
\format{
A matrix with 3 columns and 76926 rows:
\describe{
  \item{Pathway}{An MSigDB annotated pathway}
  \item{ENTREZID}{The ENTREZID of a gene belonging to the pathway}
  \item{ENSEMBL}{The ENSEMBL of a gene belonging to the pathway}
}
}
\usage{
data(path_gene_table)
}
\description{
A table of gene-pathway association. based on the pathways of MSigDB.
}
\examples{
data(path_gene_table)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{sumlogFn}
\alias{sumlogFn}
\title{The function calculate targeting score of miRNA w.r.t to a cluster
of pathways via sumlog aggregation method.}
\usage{
sumlogFn(enriches, pathways, isSelector, thresh = NULL)
}
\arguments{
\item{enriches}{a table of miRNA pathway enrichments. Universe}

\item{pathways}{queried pathways. e.g. cluster pathways}

\item{isSelector}{internal argument}

\item{thresh}{internal argument}
}
\value{
a  scoring of miRNAs in a cluster of pathways
}
\description{
The function calculate targeting score of miRNA w.r.t to a cluster
of pathways via sumlog aggregation method.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{targetScan_03}
\alias{targetScan_03}
\title{A processed list of miRNA target gene sets from the TargetScan dataset.
Each list item is a list of genes targeted by the respective miRNA family}
\format{
A list of 439 items
}
\source{
\url{http://www.targetscan.org/vert_72/}
}
\usage{
data(targetScan_03)
}
\description{
The interactions are filtered to only human interactions.
}
\details{
The interactions are filtered to have a
Cumulative weighted context++ score of < -0.3
}
\examples{
data(targetScan_03)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{reportEnrichment}
\alias{reportEnrichment}
\title{Publication-ready miRNA-Pathway Enrichment table}
\usage{
reportEnrichment(enrichmentTable)
}
\arguments{
\item{enrichmentTable}{Outputs from [miRNAPathwayEnrichment()] function}
}
\value{
A summarized miRNA-Pathway enrichment table
}
\description{
This function summarizes the outputs
}
\examples{
data(msigdb_c2)
data(targetScan_03)
eTab <- miRNAPathwayEnrichment(targetScan_03[1:20],msigdb_c2[1:20])

repTab <- reportEnrichment(eTab)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{pcxnToNet}
\alias{pcxnToNet}
\title{Creates a network out of pcxn table}
\usage{
pcxnToNet(pcxn, edgeFDR, correlationCutOff, weighted)
}
\arguments{
\item{pcxn}{pathways network edge list of pathways}

\item{edgeFDR}{FDR threshold for pathway-pathway adjusted p-values;
filter edges with adjusted p-values less than given threshold}

\item{correlationCutOff}{cut-off threshold for pathway-pathway correlation;
filter pathways with correlation less than given threshold}

\item{weighted}{True if you wish to include correlation weights in clustering}
}
\value{
enrichment analysis results
}
\description{
Creates a network out of pcxn table
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{sumlogCoverFn}
\alias{sumlogCoverFn}
\title{Internal function for modification of prioritization.}
\usage{
sumlogCoverFn(selector, coverName)
}
\arguments{
\item{selector}{a prioritzation table}

\item{coverName}{a new column name}
}
\value{
an updated scoring of miRNAs in a cluster of pathways
}
\description{
Internal function for modification of prioritization.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{methodProbBase}
\alias{methodProbBase}
\title{Outputs a table with col x, miRNA, probability of observing k
  against a random distribution of the cover of methodology}
\usage{
methodProbBase(samplingData, selector, m, nPaths = 100, coverFn = NULL)
}
\arguments{
\item{samplingData}{Random distribution data.}

\item{selector}{Table with x(miRNA) in pathway cluster and observed
k (depending on methodology).}

\item{m}{Method name.}

\item{nPaths}{Number of pathways used to generate the samplingData at
each iteration. Default is set at 100.}

\item{coverFn}{Cover of methodology function.}
}
\value{
Outputs a new selector table with col x, pval and cover.
}
\description{
Outputs a table with col x, miRNA, probability of observing k
  against a random distribution of the cover of methodology
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{miniTestsPanomiR}
\alias{miniTestsPanomiR}
\title{Readouts and datasets for minimal reproducible examples of the PanomiR.}
\format{
A list of 5:
\describe{
  \item{mini_LIHC_Exp}{a reduced expression dataset from TCGA LIHC data}
  \item{mini_LIHC_Cov}{a reduced covariates dataset from TCGA LIHC data}
  \item{miniEnrich}{a reduced table of miRNA-pathway enrichment, TargetScan.}
  \item{miniDEP}{Differentially activated pathways from reduced TCGA LIHC}
  \item{miniPCXN}{reduced representation of PCXN network}
  \item{miniPathClusts}{miniDEP mapped to miniPCXN}
}
}
\usage{
data(miniTestsPanomiR)
}
\description{
The item miniEnrich is a reduced representation of the TargetScan
For full table use miRNAPathwayEnrichment function in the package
along with msigdb_c2 and targetScan_03 datasets.
}
\details{
These datasets include reduced representation of TCGA LIHC data
for reproducing the pipeline. doi: 10.1016/j.cell.2017.05.046

A reduced representation of PCxN is provided. For full dataset and method
please refer to pcxn.org or https://doi.org/10.1371/journal.pcbi.1006042
}
\examples{
data(miniTestsPanomiR)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{pathwayGeneTab}
\alias{pathwayGeneTab}
\title{Pathway-Gene Associations}
\usage{
pathwayGeneTab(
  pathAdress = NA,
  pathwayList = NA,
  fromType = "ENTREZID",
  toType = "ENSEMBL",
  outDir = NA
)
}
\arguments{
\item{pathAdress}{Address to an RDS file containing list of pathways where
each element is a list of genes similar to GMT format.}

\item{pathwayList}{If you wish to use a list of pathways instead of a file
use this argument instead. The list must contain no NA values.}

\item{fromType}{gene annotation type used in your input data.}

\item{toType}{gene annotation type to be produced in the output.}

\item{outDir}{Address to save an RDS for a table of pathway-gene association}
}
\value{
pathExpTab Table of pathway-gene association.
}
\description{
Generates a table of pathways and genes associations.
}
\examples{
pathway1 <- c("125", "3099", "126")
pathway2 <- c("5232", "5230", "5162")
pathList <- list("Path1" = pathway1, "Path2" = pathway2)
res <- pathwayGeneTab(pathwayList = pathList)

data(msigdb_c2)
pathwayGeneTab(pathwayList = msigdb_c2[1:2])
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{getResidual}
\alias{getResidual}
\title{function to get residuals with respect to a set of covariates}
\usage{
getResidual(covariates, adjustCovars, pathSumStats)
}
\arguments{
\item{covariates}{a covariate dataframe.}

\item{adjustCovars}{covariates to adjust for}

\item{pathSumStats}{an expression matrix}
}
\value{
a matrix of adjusted expression
}
\description{
function to get residuals with respect to a set of covariates
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{getDiffExpTable}
\alias{getDiffExpTable}
\title{function to get a DE table}
\usage{
getDiffExpTable(expMat, designMat, contrastsName)
}
\arguments{
\item{expMat}{an expression matrix}

\item{designMat}{a design Matrix}

\item{contrastsName}{the contrast to perform}
}
\value{
a table of differential expression
}
\description{
function to get a DE table
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/clusterPlotter.R
\name{clusterPlot}
\alias{clusterPlot}
\title{Plots clusters of pathways with associated directionality.}
\usage{
clusterPlot(
  subNet,
  subplot = FALSE,
  topClusters = 2,
  prefix = "",
  outDir = ".",
  plotSave = TRUE
)
}
\arguments{
\item{subNet}{pathways network (edge list of pathways)}

\item{subplot}{if TRUE, store individual clusters plots and connected plots
in Figures directory of plots}

\item{topClusters}{plot figures for top x clusters}

\item{prefix}{add prefix to plots}

\item{outDir}{output directory}

\item{plotSave}{saves the plot if set true. Otherwise display}
}
\value{
a set of plots for DE-PCXN and subclusters
}
\description{
Plots clusters of pathways with associated directionality.
}
\examples{
data(miniTestsPanomiR)
clusterPlot(miniTestsPanomiR$miniPathClusts$DE_PCXN, plotSave = FALSE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{tableFromGSC}
\alias{tableFromGSC}
\title{Pathway-Gene Associations from GeneSet collections}
\usage{
tableFromGSC(gsCollection, fromType = "ENTREZID", toType = "ENSEMBL")
}
\arguments{
\item{gsCollection}{An GSEABase gene set collection object}

\item{fromType}{gene annotation type used in your input data}

\item{toType}{gene annotation type to be produced in the output}
}
\value{
A table of pathway-gene associations
}
\description{
This function enables to utilize MSigDB packages and GSEABase objects
  to incorporate customized genesets into PanomiR.
}
\examples{
data(gscExample)
tableFromGSC(gscExample)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/miRNAPrioritization.R
\name{prioritizeMicroRNA}
\alias{prioritizeMicroRNA}
\title{Prioritize miRNA}
\usage{
prioritizeMicroRNA(
  enriches0,
  pathClust,
  method = "AggInv",
  methodThresh = NULL,
  enrichmentFDR = 0.25,
  topClust = 2,
  sampRate = 1000,
  outDir = ".",
  dataDir = ".",
  saveSampling = TRUE,
  runJackKnife = TRUE,
  saveJackKnife = FALSE,
  numCores = 1,
  saveCSV = TRUE,
  prefix = "",
  autoSeed = TRUE
)
}
\arguments{
\item{enriches0}{miRNA-pathway enrichment dataset obtained from
miRNAPathwayEnrichment.}

\item{pathClust}{Pathway clusters, obtained from
MappingPathwaysClusters.}

\item{method}{Vector of methods pCut, AggInv, AggLog, sumz, sumlog.}

\item{methodThresh}{Vector of methods threshold for each method in method,
if NULL use default thresh values in method.}

\item{enrichmentFDR}{FDR cut-off calculating miRNA-pathway hits
in the input cluster based on significant enrichment readouts.}

\item{topClust}{Top x clusters to perform miRNA prioritization on.}

\item{sampRate}{Sampling rate for CLT.}

\item{outDir}{Output directory.}

\item{dataDir}{Data directory.}

\item{saveSampling}{If TRUE, saves sampling data as RDS for each cluster in
topClust in dataDir.}

\item{runJackKnife}{If TRUE, jacknifing will be performed.}

\item{saveJackKnife}{If TRUE, saves jack-knifed sampling data as RDS for each
cluster in topClust in dataDir.}

\item{numCores}{Number of CPU cores to use, must be at least one.}

\item{saveCSV}{If TRUE, saves CSV file for each cluster in topClust in
outDir.}

\item{prefix}{Prefix for all saved data.}

\item{autoSeed}{random permutations are generated based on predetermined
seeds. TRUE will give identical results in different runs.}
}
\value{
Table of miRNA and p-values, each row contains a miRNA and its
  associated p-values from the methods.
}
\description{
Outputs a table of miRNA ordered with respective p-values derived from
  method for prioritization
}
\examples{
data("miniTestsPanomiR")

prioritizeMicroRNA(enriches0 = miniTestsPanomiR$miniEnrich,
   pathClust = miniTestsPanomiR$miniPathClusts$Clustering,
   topClust = 1,
   sampRate = 50,
   method = c("aggInv"),
   saveSampling = FALSE,
   runJackKnife = FALSE,
   numCores = 1,
   saveCSV = FALSE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{samplingDataBase}
\alias{samplingDataBase}
\title{Outputs a table of sampling data(rows are miRNA and cols are samples)}
\usage{
samplingDataBase(
  enrichNull,
  selector,
  sampRate,
  fn,
  nPaths,
  samplingDataFile,
  jackKnife = FALSE,
  saveSampling,
  numCores = 1,
  autoSeed = TRUE
)
}
\arguments{
\item{enrichNull}{Enrichment dataset with x (miRNA), y (pathway) and pval
(probability of observing x in pathway cluster).}

\item{selector}{Table with x(miRNA) in pathway cluster.}

\item{sampRate}{Sampling rate.}

\item{fn}{Methodology function.}

\item{nPaths}{Number of pathways in pathway cluster.}

\item{samplingDataFile}{If file exists, load. Else, perform random sampling}

\item{jackKnife}{If TRUE, conduct sampling with one less pathway, used for
jack knifing}

\item{saveSampling}{If TRUE, data is saved.}

\item{numCores}{number of cores used}

\item{autoSeed}{random permutations are generated based on predetermined
seeds. TRUE will give identical results in different runs.}
}
\value{
Outputs of sampling data.
}
\description{
Outputs a table of sampling data(rows are miRNA and cols are samples)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{gscExample}
\alias{gscExample}
\title{Example genesets from MSigDB}
\format{
A GeneSet Collection object containing two genesets.
}
\source{
\url{http://www.gsea-msigdb.org/gsea/index.jsp}
}
\usage{
data(gscExample)
}
\description{
Example genesets from MSigDB
}
\examples{
data(gscExample)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{pathwaySummary}
\alias{pathwaySummary}
\title{Pathway Summary Statistics}
\usage{
pathwaySummary(
  exprsMat,
  pathwayRef,
  id = "ENSEMBL",
  zNormalize = FALSE,
  method = FALSE,
  deGenes = NULL,
  trim = 0,
  tScores = NULL
)
}
\arguments{
\item{exprsMat}{Gene expression matrix with row names as genes and samples
as columns.}

\item{pathwayRef}{Table of pathway-gene associations. Created from
\code{\link{pathwayGeneTab}} function.}

\item{id}{Gene annotation type in the row name of gene expression data.}

\item{zNormalize}{Normalization of pathway summary score.}

\item{method}{Choice of how to summarize gene ranks into pathway statistics.}

\item{deGenes}{List of differentially expressed genes along with t-scores.
Only necessary if working on Top 50\% summary method.}

\item{trim}{Percentage of top and bottom ranked genes to be excluded from
pathway summary statistics.}

\item{tScores}{Argument for-top-50-percent-genes method.}
}
\value{
pathExp Table of pathway activity profiles per sample.
}
\description{
Generates a table of pathway activity profiles per sample
}
\examples{
pathTab <- tibble::tribble(
 ~Pathway, ~ENTREZID,  ~ENSEMBL,
 "Path1",   "125",      "ENSG00000196616",
 "Path1",   "3099",     "ENSG00000159399",
 "Path2",   "5230",     "ENSG00000102144",
 "Path2",   "5162",     "ENSG00000168291"
 )
exprsMat <- matrix(2 * (seq_len(12)), 4, 3)
rownames(exprsMat) <- pathTab$ENSEMBL
colnames(exprsMat) <- LETTERS[seq_len(3)]
pathwaySummary(exprsMat, pathTab, method = "x2")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{linColumnFinder}
\alias{linColumnFinder}
\title{Function imported from https://github.com/th1vairam/CovariateAnalysis
Modified from http://stackoverflow.com/questions/13088770/
Function to find linearly dependednt columns of a matrix}
\usage{
linColumnFinder(mat)
}
\arguments{
\item{mat}{an input design matrix.}
}
\value{
a list of independent columns
}
\description{
Function imported from https://github.com/th1vairam/CovariateAnalysis
Modified from http://stackoverflow.com/questions/13088770/
Function to find linearly dependednt columns of a matrix
}
\examples{
data("iris")
designMat <- getDesignMatrix(iris)
linColumnFinder(designMat$design)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{jackKnifeBase}
\alias{jackKnifeBase}
\title{Outputs a table with col x (miRNA), probability of observing
k (depending on methodology) against a random distribution with
jack-knifing of the pathway cluster (removing a pathway at a time)}
\usage{
jackKnifeBase(
  selector,
  pathways,
  enrichNull,
  fn,
  jackKnifeData,
  m,
  numCores = 1
)
}
\arguments{
\item{selector}{Table with x(miRNA) in pathway cluster and observed k
(depending on methodology).}

\item{pathways}{Pathways in pathway cluster.}

\item{enrichNull}{Enrichment dataset with x (miRNA), y (pathway) and pval
(probability of observing x in pathway cluster).}

\item{fn}{Methodology function.}

\item{jackKnifeData}{Random distribution data with jack-knifing
(i.e. one less pathway)}

\item{m}{method name}

\item{numCores}{number of cores}
}
\value{
Outputs a new selector table with col x, pval_jk
}
\description{
Outputs a table with col x (miRNA), probability of observing
k (depending on methodology) against a random distribution with
jack-knifing of the pathway cluster (removing a pathway at a time)
}
